/*

Single User Dungeon for BBS's
Copyright 1998 C.A. Broadribb
Started 1993, finished ?
*/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>

#define MAXDESC 160          /* Maximum length of descriptions of the above */
#define MAXDIR 6             /* Maximum number of directions (N,S,E,W,U,D) */
#define MAXLOCATIONDESC 10   /* Maximum number of location description lines */
#define MAXMONS 30           /* Maximum number of monsters in the game at any time (1 +) */
#define MAXMONSPROTO 20      /* Maximum number of monster prototypes */
#define MAXOBJECTS 20        /* Maximum number of objects in the game at any time */
#define MAXNAME 80           /* Maximum length of names of rooms, objects, animals, spells etc */
#define MAXPLAYERS 2         /* Maximum number of players (human + computer) */
#define MAXSHOPSTUFF 20      /* Maximum number of object prototypes */
#define MAXJOBS 5            /* Number of possible occupations */
#define MAXSPECIES 2         /* Number of species that a player can be */
#define MAXSPELLS 20         /* Maximum number of spell prototypes */
#define MAXVERBS 17          /* Number of verbs used */
#define STARTLOC 1           /* Players' starting location */
#define MAXLOCATIONS 30      /* How many locations there are */
#define NO 0
#define YES 1
#define FALSE 0
#define TRUE 1
#define NONE -1
#define LOCAL 0
#define DISK -1
#define COMP -2
#define SMALLWEIGHT = .2     /* Average weight of a small item (chosen arbitrarily) */
#define MEDWEIGHT = .5       /* Average weight of a medium item (chosen arbitrarily) */
#define LARGEWEIGHT = 15     /* Average weight of a large item (chosen arbitrarily) */
#define MAXWEIGHT = 60       /* Maximum weight a player may carry (chosen arbitrarily) */
			     /* But the player has to have a bag to carry more than a few items */
			     /* A player can only carry 1 weapon, armour and clothes */
#define rnd( min, max ) (max > 0)?((rand() % (int)(((max)+1) - (min))) + (min)):0

enum directiontype {N,S,E,W,U,D};
enum jobtype {MAGICIAN,WARRIOR,THIEF,MONK,TRADER};
enum monsmoodtype {UNFRIENDLY,NEUTRAL,FRIENDLY};
enum objecttype {MISC,LIGHTWEAPON,HEAVYWEAPON,LIGHTARMOUR,HEAVYARMOUR,CLOTHES,SOAP,WAND,POTION,SCROLL,CHARM,BAG,LIGHT,SPELLBOOK,MISSILE,PRAYERBOOK,THIEFSTUFF,FOOD};
enum ownertype {NOOWNER,MONSTER,PLAYER};
enum sizetype {SMALL,MEDIUM,LARGE};
enum skillleveltype {NOSKILL,BAD,AVERAGE,GOOD};
enum speciestype {HUMAN,MONKEY};
enum terraintype {CAVE,FOREST,TUNNEL,DARKNESS,SHOP,LIGHTNESS};

char *species [MAXSPECIES] =
{
	"human",
	"monkey"
};

char *jobs [MAXJOBS] =
{
	"magician",
	"warrior",
	"priest",
	"thief",
	"trader"
};

char *verbs [MAXVERBS] =
{
	  "-",
	  "north",
	  "south",
	  "east",
	  "west",
	  "get",
	  "drop",
	  "quit",
	  "look",
	  "inventory",
	  "score",
	  "end",
	  "go",
	  "kill",
	  "eat",
	  "pray"
};

struct
{
       char name [MAXNAME];
       char desc1 [MAXDESC];
       char desc2 [MAXDESC];
       int cost;
       enum sizetype size;
       int available;         /* Number available in shop right now (0+) */
       int damage;
       int ds;                /* Defensive strength */
       enum objecttype type;
} shopstuff [MAXSHOPSTUFF];

struct
{
       char name [MAXNAME];
       char desc [MAXDESC];
       int level;             /* Skill level required to cast it */
} spells [MAXSPELLS];

struct
{
       char name [MAXNAME];
       char desc [MAXLOCATIONDESC] [80];
       enum terraintype terrain;
       int bridge;
       int map [MAXDIR];
       int fe1;      /* For future expansion */
       int fe2;      /* For future expansion */
} locations [MAXLOCATIONS];

struct
{
       int type;
       int location;         /* Location, monster or player number */
       enum ownertype owner; /* NONE, MONSTER, or PLAYER */
       int number;           /* How many the owner is carrying */
} objects [MAXOBJECTS];

struct
{
       char name [MAXNAME];
       char desc [MAXDESC];
       char attack [80];
       char walk [80];
       enum terraintype terrain;
       int life;
       int ds;                       /* max amounts */
       int skill;
       int money;
       int damage;
       enum objecttype armour;
       enum objecttype weapon;
       enum monsmoodtype mood;
} monsproto [MAXMONSPROTO];      /* Monster prototypes */

struct
{
       int type;                 /* Monster's prototype number */
       int life;
       int location;
       int mood;
       int skill;
       int damage;
       int ds;
       int money;
} mons [MAXMONS];                /* Individual monsters */

struct
{
	char loginname [MAXNAME];  /* Player's name or comp1, comp2 etc */
	char name [MAXNAME];        /* Character's name */
	char desc [MAXDESC];
	enum jobtype job;
	char sex [7];               /* Female/male/neuter */
	int age;
	int height;
	int weight;
	int money;
	int food;
	int ds;
	int damage;
	int location;
	int life;
	enum speciestype species;
	int hunger;
	int energy;
	int xp [MAXJOBS];  /* Experience in each career */
	enum skillleveltype apptitude [MAXJOBS];   /* Your apptitude for learning each career */
	enum skillleveltype skilllevel [MAXJOBS]; /* Skill level in each career (based on experience) */
	int port;
} player [MAXPLAYERS];     /* Real and computer players */

int shopmoney;
int arm,equ,mon,master,location,quit,verb,noun,which;
int playernum, numplayers, numobjects, nummons;

void adjust();
void askmoney(int,char [60],int,int);
int attack(char [80],char [80],int,int,int,int*,char [80],char [80],int,int,int,int*);
void choosenamejob(int);
void humandesc();
void monkeydesc();
int checkdd(int,int);
void checknoun(char [30]);
void checkverb(char [30]);
void clearobjectsarray ();
void clearmonsarray();
void clearlocarray();
void createmons(int);
void createchr();
void death();
void dropall();
int dropit(int);
int findvictim(float, char [30]);
int getattack(int,int,int,int,char [80],char [80]);
int getinput(char [30],char [30],char [80]);
int getit(int);
void getline(char [80],char [80]);
void getma(int,char [80],char [80]);
void help();
void hermes(char [MAXNAME]);
void describetraining();
void humantown ();
void monkeytown ();
void inventory();
void logit (char []);
void lower(int);
void monsdrop(int, int);
void monsdropall(int);
void monsmove(int);
int monsweapon(int);
void no();
void ok();
void pray();
void prompt(char [40]);
void raise(int);
void readmons();
void readobjects();
int readchr();
void readlocation();
void readshop();
void say(char [],int);
void score();
void sortequip();
void tell(char [85], char [85], int, char [85]);
void tell1andall(char [85],char [85]);
void tellallpresent(char [],int);
void writemons(char [20]);
void writeobjects(char [20]);
void writeplayer();
int yesorno(char [80]);
float playermove(int);
void assignhome (enum terraintype,int *);
int assignobject(int,int,enum ownertype,int *,int *);

void main()
{                                              /* Initialise variables   */
    int input, a;
    char temp[100];
    FILE *fp;

    randomize();
    shopmoney = mon = arm = equ = 0;
    verb = noun = which = master = location = quit = 0;
    clearobjectsarray ();
    clearmonsarray();
    clearlocarray();
    /* numplayers is the highest player number in use (0 for one player) */
    /* playernum is the player number currently active */
    numplayers = playernum = 0;
    /* Read player's name from disk instead of the following */
    strcpy(player[playernum].loginname,"Kazin Kepori");   /* change this */
    readobjects();
    if (readchr() == FALSE)
	   createchr();
    readlocation();
    readmons();                 /* Read master and ind. monsters */

/*     !!!!!!   MAIN LOOP BEGINS HERE   !!!!!!     */

    while (quit != 1)
    {
       printf("\n\nlocation = %i  ",player[playernum].location);

	      for (a = 0;a  < MAXLOCATIONDESC;a++)
		     if (strcmp(locations[player[playernum].location].desc[a],"."))
			     say(locations[player[playernum].location].desc[a],playernum);

      if (locations[player[playernum].location].terrain != DARKNESS)
      {
	      for (a = 0; a <= numplayers; a++)
	      {
		     if (player[a].location == player[playernum].location && a != playernum)
		     {
			     strcpy(temp,player[a].name);
			     if (player[a].life > 0)
			     {
				     strcat(temp," stands nearby.");
				     say(temp);
				     say(player[a].desc);
			     }
			     else
			     {
				     strcat(temp," lies on the ground, dead.");
				     say(temp);

			     }
		     }
	      }
	      for (a = 0;a <= mon;a++)
	      {
		      if (mons[a].location == (int) player[playernum].location)
		      {
			     if (mons[a].life > 0)
			     {
				       say(monsproto[mons[a].type].desc);
				       c = monsweapon(a);
				       if (c > 0)
				       {
						strcpy(temp,"It's carrying a ");
						strcat(temp,objects[c].name);
						strcat(temp,".");
						say(temp);
				       }

			     }
			     else
			     {
					strcpy(temp,"A dead ");
					strcat(temp,monsproto[mons[a].type].name);
					strcat(temp," is lying on the ground.");
					say(temp);
			     }

		      }

	      }
	      for (a = 1;a < MAXOBJECTS;a++)
	      {
		     if (objects[a].location == (int) player[playernum].location)
		     {
			     strcpy(temp,"You notice a ");
			     strcat(temp,objects[a].name);
			     strcat(temp," lying on the ground.");
			     say(temp);
		     }
	      }

	      printf("Map - N: %i  S: %i  E: %i  W: %i\n",map[(int)player[playernum].relative][1],map[(int)player[playernum].relative][2],map[(int)player[playernum].relative][3],map[(int)player[playernum].relative][4]);

       }
       if (player[playernum].hunger == 10)
       {
	       strcpy(temp,player[playernum].name);
	       strcat(temp," collapses on the ground and falls unconscious.");
	       tell1andall("You feel faint and fall to the ground.  Everything goes black.",temp);
	       quit = 1;
	       player[playernum].life = 0;
	       break;
       }
       if (player[playernum].weight > (player[playernum].life * 10) && player[playernum].carry > 2)
       {
	       b = 0;
	       say("You're feeling very weak.");
	       dropall();
       }

 /*  Second loop begins here - loop until player moves nsew  */
      while (1)
      {
					      /* Monster intelligence  */
	      for (a = 0;a <= mon; a++)
	      {
		   monsmove(a);
	      }
	      verb = 0;
	      noun = 0;
	      which = 0;                       /* get player input */
	      while (verb == 0)
	      {
		   strcpy(temp,"");
		   strcpy(temp2,"");
		   input = getinput(temp,temp2,"What do player want to do?  ");
		   checkverb(temp);
		   if (verb == 0)
			 help();
		   if (input == 2)
			 checknoun(temp2);

	      }
	      if (verb > 0 && verb < 5)           /*  N S E W  */
	      {

		  a = playermove(verb);
		  if (a == 0)                    /* Player moves  */
			break;

		  no();
		  continue;
	      }
	     if (verb == 7)                     /*  Quit   */
	     {
		  quit = 1;
		  break;
	     }
	     if (verb == 8)                      /* Look  */
	     {
		    if (which == 0)
			  break;
		    if (which == 1)
		    {
			  say(objects[noun].desc);
			  continue;
		    }
		    if (which == 7 && noun == ( (int) player[playernum].location))
		    {
			  say("You look around but find nothing.");
			  continue;
		    }
		    if (which == 8)
		    {
			  say(player[noun].desc);
			  continue;
		    }
		    no();
		    continue;
	     }




	     if (verb == 5)                       /*  get   */
	     {
		    if (which == 1)
		    {
			  strcpy(temp2,objects[noun].name);
			  c = getit(noun);
			  if (c == 0)
			  {
				 strcpy(temp,"You pick up the ");
				 strcat(temp,temp2);
				 strcat(temp,".");
				 strcpy(temp3,player[playernum].name);
				 strcat(temp3," picks up the ");
				 strcat(temp3,temp2);
				 strcat(temp3,".");
				 tell1andall(temp,temp3);
				 adjust();
			  }
			  if (c == 1)
			  {

				 say("You're carrying enough things already.");

			  }

		     }
		     if (which == 4 || which == 5)
		     {
			   say("You must say whether player want to buy or steal it.");
			   continue;
		     }
		     if (which == 6 || which == 0)
		     {
			  d = 0;
			  for (a = 1; a < MAXOBJECTS;a++)
			  {
				 strcpy(temp2,objects[a].name);
				 c = getit(a);
				 if (c == 1)
				 {
					  say("You're carrying all player can now.");
					  break;
				 }
				 if (c == 0)
				 {
					strcpy(temp,"You pick up the ");
					strcat(temp,temp2);
					strcat(temp,".");
					strcpy(temp3,player[playernum].name);
					strcat(temp3," picks up the ");
					strcat(temp3,temp2);
					strcat(temp3,".");
					tell1andall(temp,temp3);
				 }

			  }
			  adjust();
			  continue;

		     }

	     }

	     if (verb == 6)                     /*  Drop   */
	     {
		 if (which == 0)
		 {
			say("What do player want to drop?");
			continue;
		 }
		 if (which == 3)
		 {
			strcpy(temp,equip[noun].name[playernum]);
			c = dropit(noun);
			if (c < 3)
			{
				 strcpy(temp2,"You drop the ");
				 strcat(temp2,temp);
				 strcat(temp2,".");
				 strcpy(temp3,player[playernum].name);
				 strcat(temp3," drops a ");
				 strcat(temp3,temp);
				 strcat(temp3,".");
				 tell1andall(temp2,temp3);
				 sortequip();
				 adjust();
				 if (c == 1)
					  hermes(temp);
			}
			if (c == 3)
				  say("Don't drop that.  You might need it.");
			continue;
		 }

		 if (which == 6)
		 {
			 dropall();
			 continue;
		 }
		 no();
		 continue;

	     }
	     if (verb == 9)                          /* Inventory  */
	     {
		     inventory();
		     continue;
	     }

	     if (verb == 10)                                /*  Score  */
	     {
		     score();
		     continue;
	     }
	     if (verb == 11)                    /* end   */
		   exit(0);

	     if (verb == 12)                 /*  Go (requires magic wand)  */
	     {
		      b = 0;
		      for (a = 1; a <= player[playernum].carry; a++)
		      {
				if (stricmp(equip[a].name[playernum],"wand of teleportation") == 0)
					    b = a;
		      }
		      if (b == 0)
		      {
				say("You'll have to walk there yourself.");
				continue;
		      }
		      if (which != 7 || player[playernum].location == locations[noun].num)
		      {
				say("You wave the wand but nothing happens.  That's a pity.");
				continue;
		      }

		      if ((rnd(1,10)) == 1)
		      {
				say("You wave the wand.  It tingles a little then goes dead.");
				strcpy(equip[b].name[playernum],"useless wand");
				continue;
		      }
		      strcpy(temp,"You wave the wand.  It tingles in your hand and everything goes black.");
		      strcpy(temp2,player[playernum].name);
		      strcat(temp2," waves a magic wand and suddenly vanishes.");
		      tell1andall(temp,temp2);
		      strcpy(temp,"You feel yourself spinning around, then the darkness lifts.");
		      strcpy(temp2,player[playernum].name);
		      strcat(temp2," suddenly appears in a puff of smoke, holding a magic wand.");
		      player[playernum].location = locations[noun].num;
		      tell1andall(temp,temp2);
		      break;

	     }
	     if (verb == 13)                     /*  kill   */
	     {
		      if (which == 0)
		      {
			       c = findvictim(player[playernum].location,player[playernum].name);
			       if (c)
			       {
					say("You can't find anybody to attack.");
					continue;
			       }
		      }
		      if (which == 2 || which == 8)
		      {
				c = 0;
				d = getattack(8,player,which,noun,temp,temp2);
				a = getattack(which,noun,8,player,temp3,temp4);
				if (which == 2)
				{
					 e = monsproto[mons[noun].type].damage;
					 if (mons[noun].weapon)
						e = (objects[mons[noun].weapon].dd);

					 f = monsproto[mons[noun].type].ds;
					 if (mons[noun].armour)
						f = (objects[mons[noun].armour].dd - 100);

					 c = attack(temp,temp2,d,player[playernum].damage,player[playernum].ds,&player[playernum].life,temp3,temp4,a,e,f,&mons[noun].life);
					 if (c == 1)
					 {
						  monsdropall(noun);
						  player[playernum].xp[1] += mons[noun].skill;
						  strcpy(temp4,"The ");
						  strcat(temp4,monsproto[mons[noun].type].name);
						  askmoney(player,temp4,mons[noun].money,1);

					 }

				}
				else
				{

					c = attack(temp,temp2,d,player[playernum].damage,player[playernum].ds,&player[playernum].life,temp3,temp4,a,player[noun].damage,player[noun].ds,&player[noun].life);
					if (c == 1)
					{
						 player[playernum].xp[1] += player[noun].skilllevel[1];
						 askmoney(player,player[noun].name,player[noun].money,1);
						 askmoney(player,player[noun].name,player[noun].food,2);
					}
					else
					{
						 player[noun].xp[1] += player[playernum].skilllevel[1];
						 askmoney(noun,player[playernum].name,player[playernum].money,1);
						 askmoney(noun,player[playernum].name,player[playernum].food,2);
					}

				}
				if (c == 2)
				{
					dropall();
					quit = 1;
					break;
				}


		      }
		      else
			   no();
		      continue;

	     }
	     if (verb == 14)
	     {
		     if (player[playernum].food < 1)
		     {
			     say("You haven't got any food.");
			     continue;
		     }
		     player[playernum].food --;
		     player[playernum].life += 2;
		     say("You sit down to eat some delicious dried rations.");
		     continue;
	     }
	     if (verb == 15)
		     pray();







       }  /* Player input loop (second loop) ends here   */




    }  /*  Main loop ends here  */

    if (player[playernum].life < 1)
	  death();
    else
    {
	  strcpy(temp,player[playernum].name);
	  strcat(temp," vanishes in a puff of smoke.");
	  tell1andall("You quit, player coward.",temp);
    }
    writeobjects(temp);
    writemons(temp);
    writeplayer();

}   /*   Program ends here   */




/*
Give introduction (Treetop times)

Monsters:
	   maybe attack player

     ?Mail - read, write

     praying - results, xp

     stealing - money - from shopstuff, from monsters.  Check for success, mons
			attacking, decrease shopmoney/mons.money, spirit decrease
	      - objects - from shopstuff. Check for success, change avail status,
			add to equip
     buying from shopstuff - check if available,
	  check if player already have armour/weapon, increase shopmoney,
	  maybe change avail status, add to equip array, weight
     selling to shopstuff - check if it's buyable (ie in shopstuff array),
	  change avail status, decrease shopmoney, delete from your array,
	  check your DS etc, weight

 Commands for players with special privileges:
     Add - item to objects array or shopstuff array
	   weapon
	   armour           name, desc, weight, cost, DS/dam etc
	   monster
	(if there's space in array)
	Resave appropriate arrays
	   location - find a blank location.  Enter name, desc.  Resave array
*/



/*  Functions:      */


void adjust()               /* Find your DS, armour & weapons no's */
{
      int a;
      player[playernum].ds = 0;
      player[playernum].damage = 0;
      player[playernum].armour = 0;
      player[playernum].weapon = 0;
      for (a = 0; a <= player[playernum].carry; a++)
      {
	      if (equip[a].dd[playernum] > 100)
	      {
		     player[playernum].ds = (player[playernum].ds + (equip[a].dd[playernum] - 100));
		     player[playernum].armour = a;
	      }
	      if (equip[a].dd[playernum] > 0 && equip[a].dd[playernum] < 100 && equip[a].dd[playernum] > player[playernum].damage)
	      {
		     player[playernum].damage = equip[a].dd[playernum];
		     player[playernum].weapon = a;
	      }
     }
     if (player[playernum].weapon == 0)
	      player[playernum].damage = (player[playernum].skilllevel[2] + 1);
}


void askmoney(int player,char name[60],int b,int what)
{                                            /* What: 1 = cowrie shells   */
	int a;                               /*       2 = food            */
	char temp [80];                      /* b = amount                */
	char temp2 [30];                     /* Name is name of victim    */
	char temp3 [60];
	if (b)
	{
		 itoa(b,temp2,10);
		 strcpy(temp,name);
		 if (b == 1)
		 {

			if (what == 1)
			{
				strcat(temp," was carrying 1 measly cowrie shell.");
				say(temp);

			}
			else
			{
				strcat(temp," had 1 day's worth of rations.");
				say(temp);

			}
			strcpy(temp3,"Do player want it?  ");
		 }
		 else
		 {
			strcat(temp," was carrying ");
			strcat(temp,temp2);
			if (what == 1)
				strcat(temp," cowrie shells.");
			else
				strcat(temp," rations of food.");
			say(temp);
			strcpy(temp3,"Do player want them?  ");
		 }
		 strcpy(temp,"");
		 strcpy(temp2,"");
		 a = yesorno(temp3);
		 if (a)
		 {
			if (what == 1)
				player[playernum].money += b;
			else
				player[playernum].food += b;
			if (player[playernum].job == 3)
			{
				say("Robbing dead bodies does not please the gods!");
				lower(3);
			}


		 }

	}

}


					      /* Continuous fighting   */
int attack(char uattack1 [80],char itattack1 [80],int skill1,int dam1,int ds1,int *life1,char uattack2 [80],char itattack2 [80],int skill2,int dam2,int ds2,int *life2)
{
       int as, dead = 0;                     /* returns 1 if defender dies */
       while (*life1 > 0 && *life2 > 0)      /* 2 if attacker dies         */
       {                                     /* 0 if neither die           */
	       as = (skill1 + (rnd(0,5)) - ds2);
	       if (as > dam1)
		       as = dam1;
	       if (as > 0)
	       {
		      *life2 -= as;

	       }
	       if (life2 > 0)
	       {
		       as = (skill2 + (rnd(0,5)) - ds1);
		       if (as > dam2)
			       as = dam2;
		       if (as > 0)
		       {
			       *life1 -= as;


		       }

	       }

       }
       if (*life1 < 1)
       {
	     puts("attacker dies");
	     dead = 2;
       }
       else
       {
	     puts("defender dies");
	     dead = 1;
       }
	return(dead);
}

void choosenamejob(int bestapp)
{
	   int a;
	   char temp [80], temp2[80];
	   char firstname [30]; secondname [30];

	   if (player[playernum].age == 8 || player[playernum].age == 11 || player[playernum].age == 18)
		sprintf(temp,"You are an %i year old %s %s.  You are %i cm tall and weigh %i kg.",player[playernum].age,player[playernum].sex,species[player[playernum].species],player[playernum].height,player[playernum].weight);
	   else
		sprintf(temp,"You are a %i year old %s %s.  You are %i cm tall and weigh %i kg.",player[playernum].age,player[playernum].sex,species[player[playernum].species],player[playernum].height,player[playernum].weight);
	   say(temp,playernum);
	   switch (player[playernum].species)
	   {
		    case HUMAN: humandesc ();say(player[playernum].desc,playernum);humantown ();break;
		    case MONKEY: monkeydesc ();say(player[playernum].desc,playernum);monkeytown ();break;
	   }
	   say("You have just finished your basic education.  It's time to choose a career.",playernum);
	   say("Your teacher has assessed your apptitude for various careers.",playernum);
	   say("Career            Apptitude",playernum);
	   for (a = 0;a < MAXJOBS; a++)
	   {
		sprintf (temp,"%i  %-15s%s",a+1,jobs[a],player[playernum].apptitude[a]);
		say(temp,playernum);
	   }
	   sprintf(temp,"Your teacher advises you to become a %s.",jobs[bestscore]);
	   say(temp,,playernum);
	   do
		getinput(temp,temp2,"Which one do you choose?  ");
	   while (atoi(temp) < 1 || atoi(temp) > MAXJOBS);
	   player[playernum].job = atoi(temp);
	   say("The custom in this land is that when you change careers, you choose a new",playernum);
	   say("name for yourself that reflects your new social role.",playernum);
	   while (strcmp(player[playernum].name,"") == 0)
	   {
		    getinput(firstname,secondname,"What name will you be known by?  ");
		    strcat(player[playernum].name,firstname);
		    if(strcmp(secondname,"") && (strlen(player[playernum].name) + strlen(secondname)) < MAXNAME)
		    {
			    strcat(player[playernum].name," ");
			    strcat(player[playernum].name,secondname);
		    }
	   }
	   describetraining ();
	   if (player[playernum].species == MONKEY)
		player[playernum].age += 1;
	   if (player[playernum].species == HUMAN)
		player[playernum].age += 3;
}

void humandesc ()
{
	char *skincol [4] =
	{
		"fair",
		"light brown",
		"dark brown",
		"olive",
	};
	char *haircol [6] =
	{
		"fair",
		"red",
		"light brown",
		"brown",
		"dark brown",
		"black"
	};
	char *eyecol [5] =
	{
		"blue",
		"green",
		"grey",
		"brown",
		"black",
	};

	sprintf (player[playernum].desc,"You have %s skin, %s hair and %s eyes.",skincol[rnd(0,3)],haircol[rnd(0,5)],eyecol[rnd(0,4)]);
}

void monkeydesc ()
{
	char *haircol [6] =
	{
		"soft, light green",
		"course, dark green",
		"reddish-brown",
		"brown",
		"golden",
		"black"
	};
	char *eyecol [5] =
	{
		"big",
		"green",
		"brown",
		"little beady",
		"black"
	};

	sprintf (player[playernum].desc,"You have %s fur and %s eyes.",haircol[rnd(0,5)],eyecol[rnd(0,4)]);
}

int checkdd(int a,int x)               /* a = mons number, x = objectsect num.  */
{                                      /* If it's an appropriate armour or  */
       int ok = 1, small = 0;          /* weapon, pick it up & return 0     */
       if(objects[x].dd == 0)              /* else return 1                     */
	      return(1);
       if(monsproto[mons[a].type].aw == 0)
	      return(1);
       if (strstr(objects[x].name,"small") != NULL || strstr(objects[x].name,"short") != NULL)
	      small = 1;
       if( (small == 0 && objects[x].dd < 100 && monsproto[mons[a].type].aw == 1) || (small == 1 && objects[x].dd < 100 && monsproto[mons[a].type].aw == 2) )
       {
    /*          printf("checkdd:  mons no = %i mons aw = %i\n",a,monsproto[mons[a].type].aw);  */
	      objects[x].location = (a + 700);
    /*          printf("objectsect location = %i\n",objects[x].location); */
	      mons[a].weapon = x;
	      ok = 0;
       }
       return(ok);

}



void checknoun(char temp2 [30])
{
	int b = 0;
	noun = 0;
	which = 0;

	if (stricmp(temp2,"all") == 0)
	{
	      which = 6;
	      return;
	}
	for (b = 0; b <= player[playernum].carry;b++)
	{
		if (strnicmp(temp2,equip[b].name[playernum],strlen(temp2)) == 0)
		{
			noun = b;
			which = 3;
			return;
		}
	}

	for (b = 0;b < MAXOBJECTS;b++)
	{
	       if (strnicmp(temp2,objects[b].name,strlen(temp2)) == 0 && objects[b].location == player[playernum].location )
	       {
		      noun = b;
		      which = 1;
		      return;
	       }
	 }
	 for (b = 0; b <= mon;b++)
	 {
	       if (strnicmp(temp2,monsproto[mons[b].type].name,strlen(temp2)) == 0 && mons[b].location == player[playernum].location && mons[b].life > 0)
	       {
		      noun = b;
		      which = 2;
		      return;
	       }
	 }
	 for (b = 0; b <= player;b++)
	 {
	       if (strnicmp(temp2,player[b].name,strlen(temp2)) == 0 && player[b].location == player[playernum].location && b != player)
	       {
		      noun = b;
		      which = 8;
		      return;
	       }
	 }

	 for (b = 1; b < location;b++)
	 {
	       if (stricmp(temp2,locations[b].name) == 0)
	       {
		      noun = b;
		      which = 7;
		      return;
	       }
	 }

	if (player[playernum].location != 20)
	      return;
	for (b = 0; b <= equ;b++)
	{
	       if (strnicmp(temp2,shopstuff[b].name,strlen(temp2)) == 0 && shopstuff[b].avail == 1)
	       {
		      noun = b;
		      which = 4;
		      return;
	       }
	}
	for (b = 0; b <= arm; b++)
	{
	      if (strnicmp(temp2,shopstuff[b].name,strlen(temp2)) == 0 && shopstuff[b].avail == 1)
	       {
		      noun = b;
		      which = 5;
		      return;
	       }
	}

}


void checkverb(char temp [30])
{

	int b = 0;
	verb = 0;
	for (b = 1; b < MAXVERBS;b++)
	{
		if (strnicmp(temp,verbs [b],strlen(temp)) == 0)
		{
			verb = b;
			break;
		}
	}

}


void clearobjectsarray ()
{
	int loop;

	for (loop = 0; loop < MAXOBJECTS; loop++)
	{
	       objects[loop].type = -1;
	       objects[loop].location = -1;
	       objects[loop].owner = -1;
	       objects[loop].number = -1;
	}
	for (loop = 0; loop < MAXSHOPSTUFF; loop++)
	{
	       strcpy(shopstuff[loop].name,"");
	       strcpy(shopstuff[loop].desc1,"");
	       strcpy(shopstuff[loop].desc2,"");
	       shopstuff[loop].cost = -1;
	       shopstuff[loop].size = -1;
	       shopstuff[loop].available = -1;
	       shopstuff[loop].damage = shopstuff[loop].ds = shopstuff[loop].type = -1;
	}
}


void clearmonsarray()
{
	int a;

	for (a = 0;a < MAXMONSPROTO;a++)
	{
		strcpy(monsproto [a].name,"");
		strcpy(monsproto [a].desc,"");
		strcpy (monsproto [a].walk,"");
		strcpy (monsproto [a].attack,"");
		monsproto [a].terrain = -1;
		monsproto [a].life = -1;
		monsproto [a].ds = -1;
		monsproto [a].skill = -1;
		monsproto [a].money = -1;
		monsproto [a].damage = -1;
		monsproto [a].armour = -1;
		monsproto [a].mood = -1;
		monsproto [a].weapon = -1;
	}
	for (a = 0;a < MAXMONS;a++)
	{
		mons [a].type = -1;
		mons [a].life = -1;
		mons [a].location = -1;
		mons [a].mood = -1;
		mons [a].skill = -1;
		mons [a].damage = -1;
		mons [a].ds = -1;
		mons [a].money = -1;
	}
}


void clearlocarray()
{
	    int a, b;

	    for (a = 0;a < MAXLOCATIONS;a++)
	    {
		  strcpy(locations[a].name,"");
		  locations[a].terrain = -1;
		  locations[a].bridge = -1;
		  for (b = 0;b < MAXLOCATIONDESC;b++)
			  strcpy(locations[a].desc[b],".");
		  for (b = 0; b < MAXDIR; b ++)
			locations[a].map[b] = -1;
	    }
}



void createmons(int monsnum)
{
      int monstype;
      char temp [80];

   /* puts("In createmons");       */
      monstype = rnd(0,MAXMONSPROTO -1);
      mons[monsnum].type = monstype;
      mons[monsnum].life = monsproto[monstype].life + (rnd(0,20)) - 10;
      if (mons[monsnum].life < 1)
		mons[monsnum].life = 0;
      mons[monsnum].skill = monsproto[monstype].skill + (rnd(0,10)) - 5;
      if (mons[monsnum].skill < 1)
		mons[monsnum].skill = 0;
      mons[monsnum].money = (rnd(1,monsproto[monstype].money));
      mons[monsnum].mood = monsproto[monstype].mood;
      mons[monsnum].damage = monsproto[monstype].damage;
      mons[monsnum].ds = monsproto[monstype].ds;
      assignhome (monsproto[monstype].terrain,&(mons[monsnum].location));
      sprintf (temp,"A %s %s over.",monsproto[monstype].name,monsproto[monstype].walk);
      tellallpresent (temp,mons[monsnum].location);
      assignobject(monsproto[monstype].weapon,monsnum,MONSTER,&mons[monsnum].damage,NULL);
      assignobject(monsproto[monstype].armour,monsnum,MONSTER,NULL,&mons[monsnum].ds);
}

void createchr()
{
/* Create a new character (except for char name, desc, job) */
/* player[playernum].player should have been set already */
	   int a, bestapp;

	   player[playernum].species = (rnd(0,MAXSPECIES));
	   if (player[playernum].species == HUMAN)
	   {
		player[playernum].age = (rnd(18,60));
		player[playernum].height = (rnd(150,210));
		player[playernum].weight = (rnd(40,150));
	   }
	   if (player[playernum].species == MONKEY)
	   {
		player[playernum].age = (rnd(1,15));
		player[playernum].height = (rnd(50,110));
		player[playernum].weight = (rnd(20,75));
	   }
	   switch ((rnd(0,2))
	   {
		case 0:strcpy(player[playernum].sex,"female");break;
		case 1:strcpy(player[playernum].sex,"neuter");break;
		case 2:strcpy(player[playernum].sex,"male");break;
	   }
	   bestapp = 0;
	   for(a = 0;a < MAXJOBS;a++)
	   {
		  player[playernum].apptitude[a] = (rnd(0,3));
		  if (player[playernum].apptitude[a] >= player[playernum].apptitude[bestapp])
			bestapp = a;
	   }
	   choosenamejob(bestapp);
	   player[playernum].ds = 0;
	   player[playernum].damage = 1;
	   player[playernum].location = STARTLOC;
	   player[playernum].life = rnd(10,30);
	   player[playernum].hunger = 0;
	   player[playernum].energy = player[playernum].life / 2;
	   player[playernum].relativeloc = STARTLOC;
	   for (a = 0; a < MAXJOBS; a ++)
	   {
		  player[playernum].xp[a] = 0;
		  player[playernum].skilllevel[a] = NONE;
	   }
	   player[playernum].skilllevel[player[playernum].job] = BAD;
	   assignobject(FOOD,playernum,PLAYER,NULL,NULL);
	   assignobject(BAG,playernum,PLAYER,NULL,NULL);
	   player[playernum].money = (rnd(1,3));
	   switch (player[playernum].job)
	   {
		case MAGICIAN: assignobject(SPELLBOOK,playernum,PLAYER);break;
		case WARRIOR: assignobject(HEAVYWEAPON,playernum,PLAYER);break;
		case MONK: assignobject(PRAYERBOOK,playernum,PLAYER);break;
		case THIEF: assignobject(THIEFSTUFF,playernum,PLAYER);break;
		case TRADER: assignobject(SOAP,playernum,PLAYER);break;
	   }
}

void death()
{
      say("You have died.");
}


void dropall()
{
	char temp [80];
	char temp2 [80];
	char temp3 [80];
	int a, b = 0, c;
	for (a = 1;a <= player[playernum].carry;a++)
	{
		  strcpy(temp,equip[a].name[playernum]);
		  c = dropit(a);
		  if (c < 2)
		  {
			   strcpy(temp2,"You drop the ");
			   strcat(temp2,temp);
			   strcat(temp2,".");
			   strcpy(temp3,player[playernum].name);
			   strcat(temp3," drops a ");
			   strcat(temp3,temp);
			   strcat(temp3,".");
			   tell1andall(temp,temp3);
			   if (c == 1)
				     hermes(temp);
		  }

	}
	sortequip();
	adjust();
}



int dropit (int a)                           /*  0 = success               */
{                                            /*  1 = success but objectsect    */
    int c = 2, loop;                         /*      disappears            */
					     /*  2 = error                 */
					     /*  3 = don't drop valuables  */

    if (stricmp(equip[a].name[playernum],"lunch box") == 0)
	    return(3);
    if (stricmp(equip[a].name[playernum],"spell book") == 0)
	    return(3);
    if (strcmp(equip[a].name[playernum],"-"))
    {
	   c = 1;
	   for (loop = 1; loop < MAXOBJECTS; loop ++)
	   {
		  if (strcmp(objects[loop].name,"-") == 0)
		  {
			 c = 0;
			 strcpy(objects [loop].name,equip[a].name[playernum]);
			 objects [loop].weight = equip[a].weight[playernum];
			 objects [loop].location = (int) player[playernum].location;
			 objects [loop].dd = equip[a].dd[playernum];
			 break;
		  }
	   }
	   player[playernum].weight = player[playernum].weight - equip[a].weight[playernum];
	   strcpy(equip[a].name[playernum],"-");
	   equip[a].weight[playernum] = 0;
	   equip[a].dd[playernum] = 0;

    }
return(c);
}


int findvictim(float atlocation, char atname [30])
{                                        /* Find a mons/person to attack  */
					 /* 0 = success, 1 = failure      */
	int a, b, c = 0;                 /* ATNAME = name of attacker     */
					 /* ATlocation = location no of attacker  */
	noun = 9999;                     /* NOUN & WHICH are set to attackee */
	for (a = 0;a <= mon; a++)
	{
		   if (mons[a].location == atlocation && strcmp(monsproto[mons[a].type].name,atname) != 0)
		   {
			     noun = a;
			     which = 2;
			     c = 0;
			     break;

		   }
	}
	if (noun == 9999)
	{
		   for (a = 0; a <= player; a++)
		   {
			     if (player[a].location == player[playernum].location && strcmp(player[a].name,atname) != 0)
			     {
				      noun = a;
				      which = 8;
				      c = 0;
				      break;
			     }
		    }

       }
       if (noun == 9999)
		 c = 1;
return(c);
}

int getattack(int what,int num,int defwhat,int defnum,char uattack [80],char itattack [80])
{                                         /* Defwhat is defender's type     */
	  char defname [30];              /* Defnum is defender's array num */
	  char temp [80];                 /* What = type of attacker:       */
	  char temp2 [80];                /*    2 (mons) or 8 (person)      */
	  int as;                         /* num = attacker's array number  */
					  /* Returns 2 strings to describe  */
					  /*  the attack from 2 points of view*/
					  /* and the attacker's A.S.        */
	  if (defwhat == 2)
	  {
		 strcpy(defname,"the ");
		 strcat(defname,monsproto[mons[defnum].type].name);
	  }
	  else
	  {
		 strcpy(defname,player[defnum].name);
	  }
	  if (what == 2)
	  {
		 as = mons[num].skill;
		 strcpy(itattack,"The ");
		 strcat(itattack,monsproto[mons[num].type].name);
		 strcat(itattack," ");
		 strcat(itattack,monsproto[mons[num].type].attack);
	  }
	  else
	  {
		 strcpy(uattack,"You");
		 strcpy(itattack,player[num].name);
		 strcat(itattack," ");

		 if (player[num].weapon)
		 {
			as = player[num].skilllevel[1];
			strcat(uattack," hit ");
			strcpy(uattack,defname);
			strcpy(uattack,".");
			strcat(itattack," hits player.");

		 }
		 else
		 {
			as = player[num].skilllevel[2];
			getma(player[num].skilllevel[2],temp,temp2);
			strcat(uattack,temp);
			strcat(uattack,defname);
			strcat(uattack,".");
			strcat(itattack,temp2);
			strcat(itattack,"player.");
		 }

	  }
	  return(as);
}


int getinput(char word1 [],char word2[],char promptstr[])
{
/* Accepts 1 or 2 words of input and returns them in word1 and word2. */
     char inputline[30];                               /* Returns no. of words */
     int a,b,c,d,e;                                /* input (1 or 2)       */
     strcpy(word1,"");
     strcpy(word2,"");

     while (strcmp(word1,"") == 0)
     {
	   strcpy(inputline,"");
	   b = c = d = e = 0;
	   while (strcmp(inputline,"") == 0)
	   {
	       prompt(promptstr);
	       fflush(stdin);
	       gets(inputline);
	   }


	   for (a = 0; a <= strlen(inputline);a++)
	   {
	       if (inputline[a] == '\0')
		     break;
	       if (inputline[a] == ' ')
	       {
		     if (b)
			  break;
		     if (strcmp(word1,"") == 0)
			  continue;
		     b = 1;
		     continue;
		}
		if (b == 0)
		{
		     word1[d] = inputline[a];
		     d += 1;
		}
		else
		{
		     word2[e] = inputline[a];
		     e += 1;
		}
	   }

     }

     word1[d] = '\0';
     word2[e] = '\0';
     c = 1;
     if (strcmp(word2,""))
	    c += 1;
     return(c);
}

						/* Returns:               */
int getit(int a)                                /* 0 = success            */
{                                               /* 1 = carrying too much  */
      int c;
      if (objects[a].location != player[playernum].location)         /* 2 = not carrying it    */
	   return(2);
      if ( player[playernum].weight + objects[a].weight < MAXWEIGHT)
      {
	   c = 0;
	   player[playernum].weight = player[playernum].weight + objects[a].weight;
	   player[playernum].carry += 1;
	   strcpy(equip[player[playernum].carry].name[playernum],objects[a].name);
	   equip[player[playernum].carry].dd[playernum] = objects [a].dd;
	   strcpy(objects[a].name,"-");
	   strcpy(objects[a].desc,"-");
	   objects[a].weight = 0;
	   objects[a].location = 0;
	   objects[a].dd = 0;
      }
      else
	   c = 1;

return(c);
}


void getline(char temp[80], char temp4[80])      /* Input (local or remote) */
{
     prompt(temp4);
     fflush(stdin);
     while (strcmp(temp,"") == 0)
	    gets(temp);
}

void getma(int skill,char uattack [80],char itattack [80])
{                                               /*  Takes your skill and  */
      int a;                                    /*  returns a rnd attack   */
      a = (rnd(1,skill));
      if (a < 2)
      {
	     strcpy(uattack," punch ");
	     strcpy(itattack," punches ");
	     return;
      }
      if (a == 2)
      {
	     strcpy(uattack," kick ");
	     strcpy(itattack," kicks ");
	     return;
      }
      if (a == 3)
      {
	      strcpy(uattack," elbow strike ");
	      strcpy(itattack," elbow strikes ");
	      return;
      }
      strcpy(uattack," throw ");
      strcpy(itattack," throws ");


}


void help()
{
	int a;
	say("There are the commands player can use (the first letter will do):");
	for (a = 1;a < MAXVERBS;a++)
	      say(verbs[a]);

}


void hermes(char temp[MAXNAME])
{
     char temp2 [80];
     tellallpresent("Hermes, messanger of the gods, appears out of nowhere.",player[playernum].location);
     strcpy(temp2,"He picks up the ");
     strcat(temp2,temp);
     strcat(temp2," and disappears.");
     tellallpresent(temp2,player[playernum].location);
}

void describetraining()
{
/* Briefly describe the player's early training in a career and their */
/* reason for going off on an adventure. */

	 if (player[playernum].species == HUMAN)
		switch (player[playernum].job)
		{
			case MAGICIAN:say("The practise of magic is illegal in your town, but",playernum);
			say("you secretly take up study under an old mage named Shurnuth.",playernum);
			say("He teaches you much about the history, working and uses of Magic.",playernum);
			say("He gives you an old reference book containing instructions and spells.",playernum);
			say("He tells you, 'Never attempt to cast a spell that you don't fully understand.",playernum);
			say("The results could be disasterous!'",playernum);
			say("You spend three years studying basic magical skilllevel with Shurnuth",playernum);
			say("until one day you are caught and thrown out of town in disgrace.",playernum);
			say("Your possessions are taken from you, however, you manage to conceal",playernum);
			say("your spell book and take it with you.",playernum);
			break;

			case WARRIOR:say("You leave town to join the Hihun Warrior School.",playernum);
			say("You train with many different kinds of weapons for a few years.",playernum);
			say("However, to become a true warrior you must venture out into the",playernum);
			say("wilderness alone, taking only your favourite weapon, and seek your",playernum);
			say("destiny.",playernum);
			break;

			case MONK:say("You feel that your true destiny lies in serving the gods.",playernum);
			say("You leave your home and most of your posessions to travel to the",playernum);
			say("Quecheb monastery, which lies high in the mountains.",playernum);
			say("There you spend many hours every day in prayer and meditation.",playernum);
			say("Your prayer book is your only valuable posession.",playernum);
			say("After a few years you decide to leave to spread religion throughout",playernum);
			say("the land.",playernum);
			break;

			case THIEF:say("Your natural mischeviousness and complete lack of morals lead you to",playernum);
			say("become a thief.",playernum);
			say("You read every scroll on lock-picking that you can find, and practise",playernum);
			say("picking pockets and shoplifting to survive.",playernum);break;
			say("You do all right for a few years, until one day, you try to steal money",playernum);
			say("from a shop and the shopkeeper catches you in the act.  You are forced to",playernum);
			say("flee town immediately, taking only what you were carrying at the time.",playernum);
			break;

			case TRADER:say("Being greedy, you devote your life to making as much money",playernum);
			say("as possible through trading.  You ask other traders in the town to teach",playernum);
			say("you their sales secrets.",playernum);
			say("However, business is very poor and you rapidly become discouraged.",playernum);
			say("You like travelling and adventure, so after a few years you leave to",playernum);
			say("explore the land.",playernum);
			break;
		}
	 if (player[playernum].species == MONKEY)
		switch (player[playernum].job)
		{
			case MAGICIAN:say("You ask the oldest, wisest monkeys to teach you magic.",playernum);
			say("You record your knowledge in a spell book.",playernum);
			say("However, the monkeys have only a limited knowledge of magic so after",playernum);
			say("a year of study you leave to go travelling.",playernum);
			break;

			case WARRIOR:say("You join the monkey soldiers who defend the territory",playernum);
			say("and learn to use all kinds of weapons.",playernum);
			say("After a year you leave them to venture out into the land to prove",playernum);
			say("yourself.",playernum);
			break;

			case MONK:say("You decide to devote your life to serving the wishes of the spirits who",playernum);
			say("created this land.",playernum);
			say("You spend a year with the other monkey priests.  However, you feel",playernum);
			say("that the spirits are calling you out into the wildernesss, so you",playernum);
			say("leave the community and venture out alone.",playernum);
			break;

			case THIEF:say("Being a mischevious monkey, you decide you would rather steal than",playernum);
			say("work for a living.",playernum);
			say("A year or so later you are caught stealing fruit from an elderly",playernum);
			say("monkey and are thrown out of the community in disgrace.",playernum);
			break;

			case TRADER:say("You decide to make as much money as possible from buying and",playernum);
			say("selling anything and everything.",playernum);
			say("There are very limited opportunities in your community, so a year",playernum);
			say("later you leave to find a better place to work.",playernum);
			break;
		}
}

void humantown ()
{
/* Briefly describe the character's background. */
	  char temp [80];
	  switch(rnd(0,6))
	  {
		  case 0:say("You grew up in Wovoja village, on the edge of the Lucol desert.",playernum);break;
		  case 1:say("You grew up in the small town of Viniv, in the Bohosh Duchy area.",playernum);break;
		  case 2:say("You have lived on a farm on Vevos island since birth.",playernum);break;
		  case 3:say("You grew up in the fishing village of Noniz.",playernum);break;
		  case 4:say("You were raised in Zimuch city, in the Kudoz Valley.",playernum);break;
		  case 5:say("You have been living with the Ehip Kukido tribe in a small town.",playernum);break;
		  case 6:say("You have been living in a large town in the Ocheztedo rainforest.",playernum);break;
	  };
	  switch(rnd(0,9))
	  {
		case 0:say("Your family are beggers.",playernum);break;
		case 1:say("Your mother is the local witch doctor.",playernum);break;
		case 2:say("You are an orphan.",playernum);break;
		case 3:say("You were raised by your grandparents.",playernum);break;
		case 4:say("Your parents are jugglers and fortune tellers.",playernum);break;
		case 5:say("Your parents are poitical activists.",playernum);break;
		case 6:say("You were raised by a group of satanic priests.",playernum);break;
		case 7:say("Your mother is an artist and your father is an animal trainer.",playernum);break;
		case 8:say("Life is very hard there.",playernum);break;
		case 9:say("Life is very boring there.",playernum);break;
	  };
}

void monkeytown ()
{
	switch(rnd(0,7))
	{
		case 0:say("You live with your family in the jungle, far from any civilisation.",playernum);break;
		case 1:say("Your family lived a zoo in Zimuch city, but escaped to the jungle.",playernum);break;
		case 2:say("You have been living in a small monkey community in the Ocheztedo forest.",playernum);break;
		case 3:say("You live in the monkey community in Wovoja Village.",playernum);break;
		case 4:say("You live with relatives in scrubland in the Kudoz Valley.";break;
		case 5:say("You live with 15 siblings in the depths of the Fabdevi forest.";break;
		case 6:say("You roam with your family on the grasslands.",playernum);breal;
		case 7:say("You live with a monkey tribe in the haunted Ezaaw forest.",playernum);break;
	};
}

void inventory()
{
	  int a;
	  char temp [50];
	  char temp2 [80];
	  say("You are carrying:");
	  for (a = 1; a <= player[playernum].carry; a++)
		   say(equip[a].name[playernum]);
	  if (player[playernum].weapon)
		   strcpy(temp,equip[player[playernum].weapon].name[playernum]);
	  else
		   strcpy(temp,"none");
	  strcpy(temp2,"Your weapon:  ");
	  strcat(temp2,temp);
	  say(temp2);
	  itoa(player[playernum].money,temp,10);
	  strcpy(temp2,"Cowrie shells:  ");
	  strcat(temp2,temp);
	  say(temp2);
	  itoa(player[playernum].food,temp,10);
	  strcpy(temp2,"No. of days' food:  ");
	  strcat(temp2,temp);
	  say(temp2);

}


void logit (char message [])
{
	FILE *fp;

	fp = fopen ("sud.log","at");
	fprintf (fp,"%s\n",message);
	fclose (fp);
	printf ("%s\n",message);
}


void lower(int score)                           /* Lose a skill level */
{                                               /* Holiness can go negative */

	if (player[playernum].skilllevel[score] > 0 || score == 3)
		player[playernum].skilllevel[score] =- 1;
	else
		player[playernum].xp[score] = 0;
}


void monsdrop(int num, int what)
{
       char temp4 [80];
       if (objects[what].location == (num + 700))
       {
		strcpy(temp4,"The ");
		strcat(temp4,monsproto[mons[num].type].name);
		strcat(temp4," drops a ");
		strcat(temp4,objects[what].name);
		strcat(temp4,".");
		tellallpresent(temp4,objects[what].location);
		objects[what].location = mons[num].location;
       }
}


void monsdropall(int num)
{
	int a;
	for (a = 0; a < MAXOBJECTS; a++)
		 monsdrop(num,a);

}


void monsmove(int a)                             /* a = monster number  */
{
       int b = 0, c = 0, done = 0, drop = 0;
       char temp4 [80];
       if (mons[a].life < 1)
       {
		for (b = 0; b <= player; b++)
		{
			if (player[playernum].location == b)
				c = 1;
		}
		if (c == 1 || ((rnd(1,3)) != 1) )
			return;
		else
		{
			/*     puts("resurrection in progress");    */
			createmons(a);
			return;
		}

       }
       if (monsproto[mons[a].type].money != 0 || monsproto[mons[a].type].aw != 0)
       {
	      for (b = 1; b < MAXOBJECTS; b++)
	      {

		      drop = 0;
		      if(objects[b].location == (700 + a) && ((rnd(1,3)) == 1) && objects[b].dd == 0)
		      {
			      monsdrop(a,b);
			      drop = 1;
			      done = 1;
		      }
		      if (objects[b].location == mons[a].location && drop == 0 && ((rnd(1,3)) == 1))
		      {
			      if (objects[b].dd)
				      c = checkdd(a,b);
			      else
			      {
				      c = 1;
				      if (monsproto[mons[a].type].money)
						c = 0;
			      }

			      if (c == 0)
			      {
				      strcpy(temp4,"The ");
				      strcat(temp4,monsproto[mons[a].type].name);
				      strcat(temp4," picks up the ");
				      strcat(temp4,objects[b].name);
				      strcat(temp4,".");
				      tellallpresent(temp4,objects[b].location);
				      done = 1;
				      if(objects[b].dd == 0)
					      objects[b].location = (700 + a);
			      }
		      }

	      }

       }
       if (done)
	      return;
       b = (rnd(1,4));
       c = map[mons[a].location] [b];
       if (c > 0 && getblock( (float) c) == getblock( (float) mons[a].location) && (locations[c].type == locations[mons[a].location].type || locations[mons[a].location].type == 4 || locations[c].type == 4))
       {

			 done = 1;
			 strcpy(temp4,"The ");
			 strcat(temp4,monsproto[mons[a].type].name);
			 strcat(temp4," ");
			 strcat(temp4,monsproto[mons[a].type].walk);
			 strcat(temp4," away ");
			 strcat(temp4,verbs[b]);
			 strcat(temp4,".");
			 tellallpresent(temp4,mons[a].location);
			 mons[a].location = map[c][b];

			 strcpy(temp4,"A ");
			 strcat(temp4,monsproto[mons[a].type].name);
			 strcat(temp4," ");
			 strcat(temp4,monsproto[mons[a].type].walk);
			 strcat(temp4," in.");
			 tellallpresent(temp4,mons[a].location);
       }


}

int monsweapon(int monster)
{
      int loop, number = 0;
      for (loop = 1; loop < MAXOBJECTS; loop ++)
      {
	       if (objects[loop].location == (700 + monster) && objects[loop].dd > 0 && objects[loop].dd < 100)
	       {
			 number = loop;
			 break;
	       }
      }
      return(number);
}


void no()
{
     if (rnd(1,2) == 1)
	   say("That's impossible, my friend.");
     else
	   say("That be not possible.");

}


void ok()
{
   say("OK");
}

void pray()
{
	int b, job;
	int a = player[playernum].skilllevel[3];
	job = player[playernum].job;
	if (player[playernum].species == 1)
		say("You pray to the Spirits of the Dark.");
	if (player[playernum].species == 0)
	{
		if (player[playernum].life < ((player[playernum].scores[3] * 2) / 4) )
			say("Being so badly wounded, player pray to Ixeqrisha, God of Healing.");
		else
		{
			if (job == 1)
				say("You pray to Alolluwa, the Goddess of War.");
			if (job == 3 || job == 0 || job == 2)
				say("You pray to Ofegdifo, the God of Wisdom.");
			if (job == 4)
				say("You pray to Ikushfupa, the God of the Winds.");
			if (job == 5)
				say("You pray to Epohdodu, the Goddess of Light.");
		}

	}
	raise(3);
	b = 30;
	if (player[playernum].skilllevel[3] > 30)
		b = player[playernum].skilllevel[3] + 10;
	if ( (rnd(1,(b - player[playernum].skilllevel[3]))) == 1)
	{

		if (player[playernum].life < (player[playernum].scores[3] * 2))
		{
			player[playernum].life += (rnd(1,10));
			if (player[playernum].life > (player[playernum].scores[3] * 2))
				player[playernum].life = (player[playernum].scores[3] * 2);
			say("You feel stronger now.");
			return;
		}
	}
	if (player[playernum].skilllevel[3] > a)
		say("You feel a spiritual awakening.");
	else
		say("Your soul is strengthened and player feel better able to face the world.");

}

void prompt(char temp3[40])
{
	  printf("%s",temp3);
}

void raise(int num)
{
	int a;
	a = (rnd(1,5));
	player[playernum].xp[num] += a;
	if (player[playernum].xp[num] >= 40)
	{
		player[playernum].xp[num] -= 40;
		player[playernum].skilllevel[num] += 1;
	}
}


void readmons()
{
/* Read monster prototypes and also individual monsters, if any. */
/* sudmast.dat and sudmons.dat */
    FILE *fp;
    int a;

    fp = fopen ("sudmast.dat","r");
    if (fp == NULL)
    {
	   logit ("Error!  Unable to find sudmast.dat.");
	   exit(1);
    }
    master = 0;
    for (a = 1;a < MAXMONSPROTO;a++)
    {
	   if (fscanf(fp,"%[^\n]%*c",monsproto [a].name) < 1)
		break;
      /*   printf("Monster %i %s\n",a,monsproto[a].name);  */
	   fscanf (fp,"%[^\n]%*c",monsproto [a].desc);
	   fscanf (fp,"%[^\n]%*c%[^\n]%*c",monsproto [a].attack,monsproto [a].walk);
      /*   printf("desc: %s\n attack: %s\n walk %s\n",monsproto[a].desc,monsproto[a].attack,monsproto[a].walk);*/
	   fscanf (fp,"%i %i %i %i %i %i %i %i %i%*c",&monsproto [a].terrain, &monsproto [a].life,&monsproto[a].ds,&monsproto[a].skill,&monsproto[a].money,&monsproto[a].damage,&monsproto[a].armour,&monsproto[a].mood,&monsproto[a].weapon);
      /*   printf("terrain: %i life: %i ds: %i skill: %i money: %i damage: %i armour %i mood %i weapon %i\n",monsproto[a].location,monsproto[a].life,monsproto[a].ds,monsproto[a].skill,monsproto[a].money,monsproto[a].damage,monsproto[a].armour,monsproto[a].mood,monsproto[a].weapon);*/
    }
    fclose(fp);
    nummons = -1;
    a = 0;
    fp = fopen("sudmons.dat","rt");                           /* Then individual array */
    if (fp != NULL)
    {
	     while (a < (MAXMONS - 1))
	     {
		   if (fscanf(fp,"%i",&mons[a].type) < 1)
			break;
	       /*  printf("read mons %i type = %i\n",a,mons[nummons].type);  */
		   fscanf(fp,"%i %i %i %i %i %i %i\n",&mons[a].life,&mons[a].location,&mons[a].mood,&mons[a].skill,&mons[a].damage,&mons[a].ds,&mons[a].money);
	      /*   printf("location %i life %i armour %i skill %i money %i mood %i weapon %i\n",mons[a].location,mons[a].life,mons[a].armour,mons[a].skill,mons[a].money,mons[a].mood,mons[a].weapon); */
		   a ++;
	     }
	     nummons = a - 1;
    }
    fclose(fp);
    else
    {
	     for (a = 0;a < MAXMONS;a++)
		  createmons(a);
	     nummons = a;
    }
}


void readobjects()
{
/* Read the master objects (shopstuff) in sudshop.dat and, if it exists, */
/* active objects in sudobj.dat */
/* The data for the shopstuff[].available is in sudavail.dat */

    FILE *fp;
    int a, reading;

    fp = fopen("sudshop.dat","rt");
    if (fp == NULL)
    {
	 logit ("Error!  Unable to find sudshop.dat.");
	 exit(1);
    }
    for (a = 0;a < MAXSHOPSTUFF;a++)
    {
	  if (fscanf(fp,"%[^\n]%*c",shopstuff[a].name) == EOF)
		break;
	  fscanf(fp,"%[^\n]%*c%[^\n]%*c",shopstuff[a].desc1,shopstuff[a].desc2);
     /*    printf("object %i: name %s\ndesc1 %s\n desc2 %s\n",a,shopstuff[a].name,shopstuff[a].desc1,shopstuff[a].desc2); */
	  fscanf(fp,"%i %i %i %i %i %*c",&shopstuff[a].size,&shopstuff[a].cost,&shopstuff[a].damage,&shopstuff[a].ds,&shopstuff[a].type);
	  shopstuff[a].available = 0;
     /*   printf("cost %i weight %i  damage %i ds %i type %i\n",shopstuff[a].cost,shopstuff[a].size,shopstuff[a].damage,shopstuff[a].ds,shopstuff[a].type);   */
     }
     fclose(fp);
     reading = 0;
     if (fp = fopen ("sudavail.dat","rt"))
	for a = 0; a < MAXSHOPSTUFF && reading != EOF; a ++)
		reading = fscanf (fp,"%i\n",&shopstuff[a].available;
     fclose (fp);
     numobjects = -1;
     a = 0;
     if ((fp = fopen("sudobj.dat","rt"))
     {
		while (a < (MAXOBJECTS - 1))
		{
			if (fscanf(fp,"%i %i %i %i%*c",&objects[a].type,&objects[a].location,&objects[a].owner,&objects[a].number) == EOF)
				break;
			a ++;
		}
		numobjects = a - 1;
     }
     fclose (fp);
}


int readchr()
{
/* player[playernum].loginname must already be set to the name to look for */
/* If sudchr.dat exists, read it until the player's record is found or */
/* the end of the file is reached.  Return TRUE if successful, else FALSE */
     FILE *fp;
     char temp [81];
     int foundit = FALSE;

     fp = fopen("sudchr.dat","r");
     if (fp)
     {
	do
	{
	    strcpy(temp,"");
	    if (fscanf(fp,"%[^\n]%*c",temp) < 1);      /* A player's name  */
		break;
	    fscanf(fp,"%[^\n]%*c%[^\n]%*c",player[playernum].name,player[playernum].desc);
	    fscanf(fp,"%i\n%s\n%i\n%i\n%i\n%i\n%i\n%i\n",&player[playernum].job,player[playernum].sex,&player[playernum].money,&player[playernum].ds,&player[playernum].damage,&player[playernum].location,&player[playernum].life,&player[playernum].species);
	    fscanf(fp,"%i\n%i\n",&player[playernum].hunger,&player[playernum].energy);
	    for (a = 0; a < MAXJOBS; a++)
		 fscanf(fp,"%i\n",&player[playernum].xp[a]);
	    for (a = 0;a < MAXJOBS;a++)
		fscanf(fp,"%i\n",&player[playernum].apptitude[a]);
	    for (a = 0;a < MAXJOBS;a++)
		 fscanf(fp,"%i\n",&player[playernum].skilllevel[a]);
	    fscanf(fp,"%i\n%i\n",&player[playernum].port,&player[playernum].age);
	    fscanf(fp,"%i\n%i\n",&player[playernum].height,&player[playernum].weight);
	  } while (strcmp(player[playernum].loginname,temp));
	  foundit = TRUE;
      }
      fclose(fp);
      return (foundit);
}


void readlocation()
{
    FILE *fp;
    int a, desc;

    fp = fopen("sudloc.dat","rt");
    if (fp == NULL)
    {
	 logit ("Error!  Unable to find sudloc.dat");
	 exit(1);
    }
    for (a=0;a<MAXLOCATIONS;a++)
    {
	  if (fscanf(fp,"%[^\n]%*c",locations[a].name) == EOF)
		break;
   /*     printf("location:  %s\n",locations[a].name);    */
	  fscanf(fp,"%i%i%*c",&locations[a].terrain,&locations[a].bridge);
   /*     printf("%i %i\n",locations[a].terrain,locations[a].bridge);  */
	  for (desc=0;desc<MAXLOCATIONDESC;desc++)
	  {
		fscanf(fp,"%[^\n]%*c",locations[a].desc[desc]);
	 /*     printf("desc %i: %s\n",b,locations[a].desc[desc]);   */
	  }
	  for (desc=0; desc<MAXDIR;desc++)
	      fscanf(fp,"%i",&locations[a].map[desc]);
	  getc(fp);
    /*    printf("Map N %i S %i E %i W %i U %i D %i\n",map[a][0],map[a][1],map[a][2],map[a][3],map[a][4],map[a][5]); */
	  fscanf(fp,"%*i%*i%*c");
     }
     fclose(fp);
}



void say (char string[], int playernum)
{
	     int where;
	     where = player[playernum].port;

	     if (where == LOCAL)
			printf("%s\n",string);
	     if (where == DISK)
	     {
		/* Put on disk for other nodes */
	     }
	     if (where > 0)
	     {
		/*  Put out through com port WHERE  */
	     }
}



void score()
{
	char skillname [MAXJOBS] [13] =
	{
		"Magic",
		"Fighting",
		"Holiness",
		"Thieving",
		"Trading",
	};
	char playerdesc [80];

	sprintf(playerdesc,"Your name is %s.",player[playernum].name);
	say(playerdesc,playernum);
	sprintf(playerdesc,"You are a %s %s.",player[playernum].sex,species[player[playernum].species]);
	say(playerdesc,playernum);
	sprintf(playerdesc,"You're a %s.",jobs[player[playernum].job]);
	say(playerdesc,playernum);
	say("Your characteristics:");
	sprintf(playerdesc,"Life points:   %i.",player[playernum].life);
	say(playerdesc,playernum);
	say("Your skill levels:");
	for (a = 0; a < MAXJOBS; a++)
	{
		sprintf(playerdesc,"%s:  %15i",skillname[a],player[playernum].skilllevel[a]);
		say(playerdesc,playernum);
	}
}

void sortequip()
{
      int a;

      player[playernum].carry = 0;
      for (a = 1; a < (maxequip - 1);  a++)
      {
	   if (strcmp(equip[a].name[playernum],"-") == 0 && strcmp(equip[a+1].name[playernum],"-") != 0)
	   {
		 if (player[playernum].weapon == (a + 1))
			player[playernum].weapon = a;
		 if (player[playernum].armour == (a + 1))
			player[playernum].armour = a;

		 strcpy(equip[a].name[playernum],equip[a+1].name[playernum]);
		 equip[a].weight[playernum] = equip[a+1].weight[playernum];
		 equip[a].dd[playernum] = equip[a+1].dd[playernum];

		 strcpy(equip[a+1].name[playernum],"-");
		 equip[a+1].weight[playernum] = 0;
		 equip[a+1].dd[playernum] = 0;
	   }

      }
      for (a = 1; a < maxequip; a++)
      {
		if (strcmp(equip[a].name[playernum],"-") )
			player[playernum].carry = a;
      }
}




void tell(char msg[85], char someone[85], int num2, char others[85])
{                                      /* MSG = msg to the initiator.  */
				       /* SOMEONE = msg to reveiver (num2). */
	      int loop;                /* OTHERS = msg to anyone else present. */
				       /* Leave msgs blank if not needed.   */
	      say (msg,player);           /* If no receiver, set num2 to player.  */

	      if (strcmp(someone,""))
	      {
			  say (someone,num2);
	      }

	      if (strcmp(others,""))
	      {
			 for (loop = 0; loop <= totalplayers; loop ++)
			 {
				  if (loop != player && loop != num2 && player[loop].location == player[playernum].location)
					 say(others,loop);
			 }
	      }

}


void tell1andall(char playermsg [], char othersmsg[])
{
/* Tell playermsg to the active player and othersmsg to all other players present. */

	      int loop;

	      say (playermsg,playernum);
	      for (loop = 0; loop < numplayers; loop ++)
	      {
			if (loop != playernum && player[loop].location == player[playernum].location)
				say(othersmsg,loop);
	      }
}

void tellallpresent(char msg [], int location)
{
/* Say msg to all players in the location */

	      int loop;

	      for (loop = 0; loop < numplayers; loop ++)
	      {
			 if (player[loop].location == location)
				say(msg,loop);
	      }
}


void writemons(char block [20])
{
	FILE *fp;
	char temp [9];
	char temp3 [3];
	int a = 0,b = 0;

	fp = fopen(block,"w");
	if (fp == NULL)
		 exit(2);
	for (a = 0;a <= mon;a++)
	{
		/*  printf("Writing %i %i %i %i %i %i %i %i\n",mons[a].type,mons[a].location,mons[a].life,mons[a].armour,mons[a].skill,mons[a].money,mons[a].mood,mons[a].weapon); */
		 fprintf(fp,"%i %i %i %i %i %i %i %i\n",mons[a].type,mons[a].location,mons[a].life,mons[a].armour,mons[a].skill,mons[a].money,mons[a].mood,mons[a].weapon);
	}
	fprintf(fp,"0 0 0 0 0 0 0 0\n");
	fclose(fp);

}


void writeobjects(char block [20])
{
	FILE *fp;
	int a;
	fp = fopen(block,"w");
	if (fp == NULL)
	      exit(2);
     /*   puts("writing objects");      */
	for (a = 1;a < MAXOBJECTS;a++)
	{
	    /*    printf("%s\n%s\n",objects[a].name,objects[a].desc);  */
		fprintf(fp,"%s\n%s\n",objects[a].name,objects[a].desc);
	  /*      printf("%i %i %i\n",objects[a].weight,objects[a].location,objects[a].dd);   */
		fprintf(fp,"%i %i %i\n",objects[a].weight,objects[a].location,objects[a].dd);
	}
	fprintf(fp,"***\n");
	fclose(fp);
}

void writeplayer()
{
     FILE *fp,*of;
     char temp [30];
     char temp2 [30];
     char temp3 [120];
     int a = 0, b = 0, c = 0, flag = 0,end = 0,hey = 1,gotplayer = 0;
     float d = 0;
     strcpy(temp,"");
     strcpy(temp2,"");
     strcpy(temp3,"");
     flag = rename("player.dat","player.old");
     if (flag)
     {
	   end = 1;
	/*   puts("no existing player.dat file");   */
     }
     else
     {
	  /*  puts("reading old player file");  */
	    of = fopen("player.old","rt");
	    rewind(of);
     }
     fp = fopen("player.dat","wt");
     if (fp == NULL)
	  exit(1);

     do
     {
	 if (flag == 0)
	 {
		fscanf(of,"%[^\n]",temp);           /* read a player name  */
		getc(fp);
		if (strstr(temp,"***") != NULL || strcmp(temp,"") == 0 || strcmp(temp,"-") == 0)
		{
		       if (gotplayer == 1)
			      break;
		       else
			      end = 1;
		}
	 }
     /*    printf("read the name %s.  Your name is %s\n",temp,player[playernum].loginname); */
	 a = strncmp(temp,player[playernum].loginname,strlen(temp));
     /*    printf("compare = %i\n",a);      */
	 if (strnicmp(temp,player[playernum].loginname,strlen(temp)) == 0 || end == 1)
	 {
	      gotplayer = 1;
	      hey = 0;
	      fprintf(fp,"%s\n%s\n%s\n",player[playernum].loginname,player[playernum].name,player[playernum].desc);
	      fprintf(fp,"%i\n%s\n%i\n%i\n%i\n%i\n%f\n%i\n%i\n%i\n%i\n%i\n",player[playernum].job,player[playernum].sex,player[playernum].money,player[playernum].food,player[playernum].ds,player[playernum].damage,player[playernum].location,player[playernum].life,player[playernum].species,player[playernum].weapon,player[playernum].armour,player[playernum].weight);
	      fprintf(fp,"%i\n%i\n%f\n",player[playernum].hunger,player[playernum].carry,player[playernum].relative);
	      for (a = 0; a < 6; a++)
		       fprintf(fp,"%i\n",player[playernum].xp[a]);          /*  your xp     */
	      for (a = 0;a < 6;a++)
		       fprintf(fp,"%i\n",player[playernum].scores[a]);      /* your int. score etc */
	      for (a = 0;a < 6;a++)
		       fprintf(fp,"%i\n",player[playernum].skilllevel[a]);     /* your skill points */
	      for (a = 1;a < maxequip;a++)
	      {
		       if (strcmp(equip[a].name[playernum],"") == 0)
			      strcpy(equip[a].name[playernum],"-");

		       fprintf(fp,"%s\n",equip[a].name[playernum]);   /* your equip  */


		       fprintf(fp,"%i\n%i\n",equip[a].weight[playernum],equip[a].dd[playernum]);
	      }
	 /*   puts("written your data");  */
	      if (end == 1)
	      break;
	 }
     /*    printf("Hey = %i (0 = your data)\n",hey); */
	 if (hey)
	       fprintf(fp,"%s\n",temp);
	 fscanf(of,"%[^\n]",temp);
	 if (hey)
	       fprintf(fp,"%s\n",temp);
	 fscanf(of,"%[^\n]",temp3);
	 if (hey)
	       fprintf(fp,"%s\n",temp3);
	 fscanf(of,"%i\n%s\n",&b,temp2);
     /*    printf("%i\n%s\n",b,temp2);  */
	 if (hey)
	       fprintf(fp,"%i\n%s\n",b,temp2);
	 for (a = 0; a <4; a++)
	 {
		  fscanf(of,"%i\n",&c);
	  /*        printf("%i\n",c);        */
		  if (hey)
			 fprintf(fp,"%i\n",c);
	 }
	 fscanf(of,"%f\n",&d);
	 if (hey)
	       fprintf(fp,"%f\n",d);
	 for (a = 0; a < 7; a++)
	 {
		  fscanf(of,"%i\n",&c);
	    /*      printf("%i\n",c);      */
		  if (hey)
			fprintf(fp,"%i\n",c);
	 }
	 fscanf(of,"%f\n",&d);
	 if (hey)
		fprintf(fp,"%f\n",d);
	 for (a = 0; a < 18; a++)
	 {
		  fscanf(of,"%i\n",&c);
	   /*       printf("%i\n",c);        */
		  if (hey)
			fprintf(fp,"%i\n",c);
	 }
	 for (a = 1;a < maxequip;a++)
	 {
		  fscanf(of,"%[^\n]",temp2);
		  getc(of);
	     /*     printf("number %i... name %s\n",a,temp2);  */
		  if (hey)
			fprintf(fp,"%s\n",temp2);

		  fscanf(of,"%i\n%i\n",&b,&c);
	     /*     printf("%i\n%i\n",b,c);         */
		  if (hey)
			fprintf(fp,"%i\n%i\n",b,c);
	 }
	 if (hey == 0)
		  hey = 1;
  /*              puts("at end of do loop");                  */
     } while (1);
 /*    puts("finished loop"); */
     if (flag == 0)
     {
	    fclose(of);
	    remove("player.old");
     }
     fprintf(fp,"***\n");
     fclose(fp);
}


int yesorno(char temp3[80])               /* Get yes or no answer.  1 = yes */
{                                         /* 0 = no.                        */
       char temp[30];
       char temp2[30];
       int a = 0;
       strcpy(temp,"");
       strcpy(temp2,"");
       while (strnicmp(temp,"y",1) != 0 && strnicmp(temp,"n",1) != 0)
	       getinput(temp,temp2,temp3);
       if (strnicmp(temp,"y",1) == 0)
	       a = 1;
       return(a);
}


float playermove(int verb)                     /* Returns 0 if ok           */
{                                           /*   or 1 if impossible      */
     int b = 0, ok = 1;
     float a = 0;
     char temp [13];
     char temp2 [13];
     char temp3 [3];
     strcpy(temp,"");
     strcpy(temp2,"");
     strcpy(temp3,"");
     if (map [(int) player[playernum].relative] [verb] > 0)
     {
	   ok = 0;
	   a = (float) map [(int) player[playernum].relative] [verb];
	   if ( getblock(a) != getblock(player[playernum].location))
	   {

		 writemons(temp);                    /* Write mons & objects */
		 writeobjects(temp);
		 player[playernum].location = a;                  /* Change locations  */
		 readmons(temp,temp2);
		 readobjects();
		 readlocation();
	   }
	   if (player[playernum].location != a)
		 player[playernum].location = a;
     }
     return(ok);
}


void assignhome (enum terraintype terrain, int *location)
{
/* Set location to a location of the given terrain. */


	int loc;

	*location = -1;
	for (loc = 0; loc < MAXLOCATIONS; loc ++)
	{
		if (locations[loc].terrain == terrain)
		{
			if (*location == -1 || (rnd (0,2)) == 1)
				*location = loc;
		}
	}
}

int assignobject(int objecttype,int holdernum,enum ownertype owner,int *damage,int *ds)
{
/* Create a new object of the type objecttype and give it to holdernum */
/* Return the object number. */
/* Adds the damage and/or defensive strength of the object.
/* If there's no room for new objects in the objects array then returns NONE */

	int loop, foundone = FALSE, whichobjnum = -1, whichobj = -1;

	/* Find a spare spot in the objects array */
	for (loop = 0; loop < (MAXOBJECTS - 1) && objects[loop].type > -1; loop ++)
		;
	if (objecttype != NONE && loop < (MAXOBJECTS - 1))
	{
		whichobjnum = loop;
		if (whichobjnum > numobjects)
		     numobjects = whichobjnum;
		/* Find an appropriate object (chosen randomly) */
		for (loop = 0; loop < MAXSHOPSTUFF; loop ++)
		{
			if (shopstuff[loop].type == objecttype)
			{
				if (!foundone || (rnd(0,2)) == 1)
				{
					foundone = TRUE;
					whichobj = loop;
				}
			}
		}
		if (damage)
			*damage += shopstuff[whichobj].damage;
		if (ds)
			*ds += shopstuff[whichobj].ds;
		objects[whichobjnum].type = whichobj;
		objects[whichobjnum].location = holdernum;
		objects[whichobjnum].owner = owner;
		objects[whichobjnum].number = 1;
	}
	return (whichobjnum);
}
