/*

Single User Dungeon for BBS's
Copyright 1993 - 1997 C.A. Broadribb

*/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>

#define MAXDESC 160          /* Maximum length of descriptions of the above */
#define MAXDIR 6             /* Maximum number of directions (N,S,E,W,U,D) */
#define MAXLOCATIONDESC 10   /* Maximum number of location description lines */
#define MAXMONS 30           /* Maximum number of monsters in the game at any time (1 +) */
#define MAXMONSPROTO 20      /* Maximum number of monster prototypes */
#define MAXOBJECTS 20        /* Maximum number of objects in the game at any time */
#define MAXNAME 80           /* Maximum length of names of rooms, objects, animals, spells etc */
#define MAXPLAYERS 2         /* Maximum number of players (human + computer) */
#define MAXSHOPSTUFF 20      /* Maximum number of object prototypes */
#define MAXJOBS 5            /* Number of player skills */
#define MAXSPECIES 1         /* Number of species that a player can be */
#define MAXSPELLS 20         /* Maximum number of spell prototypes */
#define MAXVERBS 17          /* Number of verbs used */
#define MAXWEIGHT 50         /* Maximum weight that a player can carry */
#define STARTLOC 1           /* Players' starting location */
#define MAXLOCATIONS 30       /* How many locations there are */
#define NO 0
#define YES 1
#define FALSE 0
#define TRUE 1
#define NONE -1;
#define rnd( min, max ) (max > 0)?((rand() % (int)(((max)+1) - (min))) + (min)):0

enum locationtype (CAVE,FOREST,TUNNEL,DARK,SHOP);
enum monsmoodtype (UNFRIENDLY,NEUTRAL,FRIENDLY);
enum speciestype (HUMAN,MONKEY);
enum jobtype (MAGICIAN,WARRIOR,THIEF,MONK,TRADER);
enum apptitudetype (BAD,AVERAGE,GOOD);
enum directiontype (N,S,E,W,U,D);
enum ownertype (NONE,MONSTER,PLAYER);
enum terraintype (CAVE,FOREST,TUNNEL,DARK,SHOP);
enum objecttype (FOOD,LIGHTWEAPON,HEAVYWEAPON,LIGHTARMOUR,HEAVYARMOUR,CLOTHES,SOAP,WAND,POTION,SCROLL,CHARM,BAG,LIGHT,OIL);

char *species [MAXSPECIES] =
{
	"human",
	"monkey"
};

char *jobs [MAXJOBS] =
{
	"magician",
	"warrior",
	"priest",
	"thief",
	"trader"
};

char *verbs [MAXVERBS] =
{
	  "-",
	  "north",
	  "south",
	  "east",
	  "west",
	  "get",
	  "drop",
	  "quit",
	  "look",
	  "inventory",
	  "score",
	  "end",
	  "go",
	  "kill",
	  "eat",
	  "pray"
};

struct
{
       char name [MAXNAME];
       char desc1 [MAXDESC];
       char desc2 [MAXDESC];
       int cost;
       int weight;
       int available;         /* Number available in shop right now (0+) */
       int damage;
       int ds;
       enum objecttype type;
} shopstuff [MAXSHOPSTUFF];

struct
{
       char name [MAXNAME];
       char desc [MAXDESC];
       int level;             /* Skill level required to cast it */
} spells [MAXSPELLS];

struct
{
       char name [MAXNAME];
       char desc [MAXLOCATIONDESC] [80];
       enum terraintype terrain;
       int bridge;
       int map [MAXDIR];
       int fe1;      /* For future expansion */
       int fe2;      /* For future expansion */
} locations [MAXLOCATIONS];

struct
{
       int type;
       int location;         /* Location, monster or player number */
       enum ownertype owner; /* NONE, MONSTER, or PLAYER */
} objects [MAXOBJECTS];

struct
{
       char name [MAXNAME];
       char desc [MAXDESC];
       char attack [80];
       char walk [80];
       enum terraintype terrain;
       int life;
       int ds;                       /* max amounts */
       int skill;
       int money;
       int damage;
       enum objecttype armour;
       enum objecttype weapon;
       enum monsmoodtype mood;
} monsproto [MAXMONSPROTO];      /* Monster prototypes */

struct
{
       int type;                 /* Monster's prototype number */
       int life;
       int location;
       int mood;
       int skill;
       int armournum;
       int weaponnum;
       int money;
} mons [MAXMONS];                /* Individual monsters */

struct
{
	char playername [MAXNAME];  /* Player's name or comp1, comp2 etc */
	char name [MAXNAME];        /* Character's name */
	char desc [MAXDESC];
	enum jobtype job;
	char sex [7];               /* Female/male/neuter */
	int age;
	int height;
	int weight;
	int money;
	int food;
	int ds;
	int damage;
	int location;
	int life;
	enum speciestype species;
	int weapon;
	int armour;
	int weight;
	int hunger;
	int energy;
	int xp [MAXJOBS];  /* Experience in each job */
	int apptitude [MAXJOBS];   /* Your innate apptitude for each job */
	int skills [MAXJOBS]; /* Skill level in each job (based on experience) */
	int port;
} player [MAXPLAYERS];     /* Real and computer players */

int shopmoney;
int arm,equ,mon,master,location,quit,verb,noun;
int playernum, numplayers, numobjects, nummons;

void adjust();
void askmoney(int,char [60],int,int);
int attack(char [80],char [80],int,int,int,int*,char [80],char [80],int,int,int,int*);
void choosenamejob(int);
void humandesc();
void monkeydesc();
int checkdd(int,int);
void checknoun(char [30]);
void checkverb(char [30]);
void clearobjectsarray ();
void clearmonsarray();
void clearlocarray();
void createmons(int);
void createchr();
void death();
void dropall();
int dropit(int);
int findvictim(float, char [30]);
int getattack(int,int,int,int,char [80],char [80]);
int getinput(char [30],char [30],char [80]);
int getit(int);
void getline(char [80],char [80]);
void getma(int,char [80],char [80]);
float getrelative (float);
void help();
void hermes(char [MAXNAME]);
void describetraining();
void humantown ();
void monkeytown ();
void inventory();
void lower(int);
void monsdrop(int, int);
void monsdropall(int);
void monsmove(int);
int monsweapon(int);
void no();
void ok();
void pray();
void prompt(char [40]);
void raise(int);
void readextra();
void readmons()
void readobjects();
int readchr();
void readlocation(char [30]);
void readshop();
void say(char [80],int);
void score();
void sortequip();
void tell(char [85], char [85], int, char [85]);
void tell1a(char [85],char [85]);
void tellallpresent(char [],int);
void tell1(char [85]);
void writemons(char [20]);
void writeobjects(char [20]);
void writeplayer();
int yesorno(char [80]);
float playermove(int);
void assignhome (enum terraintype,int *);
int assignobject(int,int,enum ownertype);

void main()
{                                              /* Initialise variables   */
    int input;
    FILE *fp;

    randomize();
    shopmoney = mon = arm = equ = 0;
    verb = noun = which = master = location = quit = 0;
    clearobjectsarray ();
    clearmonsarray();
    clearlocarray();
    /* numplayers is the highest player number in use (0 for one player) */
    /* playernum is the player number currently active */
    numplayers = playernum = 0;
    /* Read player's name from disk instead of the following */
    strcpy(player[playernum].playername,"Kazin Kepori");   /* change this */
    readobjects();
    if (readchr() == FALSE)
	   createchr();
    readlocation();
    readmons();                 /* Read master and ind. monsters */
    readextra();

/*     !!!!!!   MAIN LOOP BEGINS HERE   !!!!!!     */

    while (quit != 1)
    {
       player[playernum].relative = getrelative(player[playernum].location);
       printf("\n\n real location no = %f  fake location no = %f\n",player[playernum].location,player[playernum].relative);

	      for (a = 0;a  < MAXLOCATIONDESC;a++)
	      {
		     if (strcmp(locations[(int)player[playernum].relative].desc[a],"."))
			     tell1(locations[(int)player[playernum].relative].desc[a]);

	      }
      if (locations[(int)player[playernum].relative].type != 4)
      {
	      for (a = 0; a <= player; a++)
	      {
		     if (player[a].location == player[playernum].location && a != player)
		     {
			     strcpy(temp,player[a].name);
			     if (player[a].life > 0)
			     {
				     strcat(temp," stands nearby.");
				     tell1(temp);
				     tell1(player[a].desc);
			     }
			     else
			     {
				     strcat(temp," lies on the ground, dead.");
				     tell1(temp);

			     }
		     }
	      }
	      for (a = 0;a <= mon;a++)
	      {
		      if (mons[a].location == (int) player[playernum].location)
		      {
			     if (mons[a].life > 0)
			     {
				       tell1(monsproto[mons[a].type].desc);
				       c = monsweapon(a);
				       if (c > 0)
				       {
						strcpy(temp,"It's carrying a ");
						strcat(temp,objects[c].name);
						strcat(temp,".");
						tell1(temp);
				       }

			     }
			     else
			     {
					strcpy(temp,"A dead ");
					strcat(temp,monsproto[mons[a].type].name);
					strcat(temp," is lying on the ground.");
					tell1(temp);
			     }

		      }

	      }
	      for (a = 1;a < MAXOBJECTS;a++)
	      {
		     if (objects[a].location == (int) player[playernum].location)
		     {
			     strcpy(temp,"You notice a ");
			     strcat(temp,objects[a].name);
			     strcat(temp," lying on the ground.");
			     tell1(temp);
		     }
	      }

	      printf("Map - N: %i  S: %i  E: %i  W: %i\n",map[(int)player[playernum].relative][1],map[(int)player[playernum].relative][2],map[(int)player[playernum].relative][3],map[(int)player[playernum].relative][4]);

       }
       if (player[playernum].hunger == 10)
       {
	       strcpy(temp,player[playernum].name);
	       strcat(temp," collapses on the ground and falls unconscious.");
	       tell1a("You feel faint and fall to the ground.  Everything goes black.",temp);
	       quit = 1;
	       player[playernum].life = 0;
	       break;
       }
       if (player[playernum].weight > (player[playernum].life * 10) && player[playernum].carry > 2)
       {
	       b = 0;
	       tell1("You're feeling very weak.");
	       dropall();
       }

 /*  Second loop begins here - loop until player moves nsew  */
      while (1)
      {
					      /* Monster intelligence  */
	      for (a = 0;a <= mon; a++)
	      {
		   monsmove(a);
	      }
	      verb = 0;
	      noun = 0;
	      which = 0;                       /* get player input */
	      while (verb == 0)
	      {
		   strcpy(temp,"");
		   strcpy(temp2,"");
		   input = getinput(temp,temp2,"What do player want to do?  ");
		   checkverb(temp);
		   if (verb == 0)
			 help();
		   if (input == 2)
			 checknoun(temp2);

	      }
	      if (verb > 0 && verb < 5)           /*  N S E W  */
	      {

		  a = playermove(verb);
		  if (a == 0)                    /* Player moves  */
			break;

		  no();
		  continue;
	      }
	     if (verb == 7)                     /*  Quit   */
	     {
		  quit = 1;
		  break;
	     }
	     if (verb == 8)                      /* Look  */
	     {
		    if (which == 0)
			  break;
		    if (which == 1)
		    {
			  tell1(objects[noun].desc);
			  continue;
		    }
		    if (which == 7 && noun == ( (int) player[playernum].location))
		    {
			  tell1("You look around but find nothing.");
			  continue;
		    }
		    if (which == 8)
		    {
			  tell1(player[noun].desc);
			  continue;
		    }
		    no();
		    continue;
	     }




	     if (verb == 5)                       /*  get   */
	     {
		    if (which == 1)
		    {
			  strcpy(temp2,objects[noun].name);
			  c = getit(noun);
			  if (c == 0)
			  {
				 strcpy(temp,"You pick up the ");
				 strcat(temp,temp2);
				 strcat(temp,".");
				 strcpy(temp3,player[playernum].name);
				 strcat(temp3," picks up the ");
				 strcat(temp3,temp2);
				 strcat(temp3,".");
				 tell1a(temp,temp3);
				 adjust();
			  }
			  if (c == 1)
			  {

				 tell1("You're carrying enough things already.");

			  }

		     }
		     if (which == 4 || which == 5)
		     {
			   tell1("You must say whether player want to buy or steal it.");
			   continue;
		     }
		     if (which == 6 || which == 0)
		     {
			  d = 0;
			  for (a = 1; a < MAXOBJECTS;a++)
			  {
				 strcpy(temp2,objects[a].name);
				 c = getit(a);
				 if (c == 1)
				 {
					  tell1("You're carrying all player can now.");
					  break;
				 }
				 if (c == 0)
				 {
					strcpy(temp,"You pick up the ");
					strcat(temp,temp2);
					strcat(temp,".");
					strcpy(temp3,player[playernum].name);
					strcat(temp3," picks up the ");
					strcat(temp3,temp2);
					strcat(temp3,".");
					tell1a(temp,temp3);
				 }

			  }
			  adjust();
			  continue;

		     }

	     }

	     if (verb == 6)                     /*  Drop   */
	     {
		 if (which == 0)
		 {
			tell1("What do player want to drop?");
			continue;
		 }
		 if (which == 3)
		 {
			strcpy(temp,equip[noun].name[playernum]);
			c = dropit(noun);
			if (c < 3)
			{
				 strcpy(temp2,"You drop the ");
				 strcat(temp2,temp);
				 strcat(temp2,".");
				 strcpy(temp3,player[playernum].name);
				 strcat(temp3," drops a ");
				 strcat(temp3,temp);
				 strcat(temp3,".");
				 tell1a(temp2,temp3);
				 sortequip();
				 adjust();
				 if (c == 1)
					  hermes(temp);
			}
			if (c == 3)
				  tell1("Don't drop that.  You might need it.");
			continue;
		 }

		 if (which == 6)
		 {
			 dropall();
			 continue;
		 }
		 no();
		 continue;

	     }
	     if (verb == 9)                          /* Inventory  */
	     {
		     inventory();
		     continue;
	     }

	     if (verb == 10)                                /*  Score  */
	     {
		     score();
		     continue;
	     }
	     if (verb == 11)                    /* end   */
		   exit(0);

	     if (verb == 12)                 /*  Go (requires magic wand)  */
	     {
		      b = 0;
		      for (a = 1; a <= player[playernum].carry; a++)
		      {
				if (stricmp(equip[a].name[playernum],"wand of teleportation") == 0)
					    b = a;
		      }
		      if (b == 0)
		      {
				tell1("You'll have to walk there yourself.");
				continue;
		      }
		      if (which != 7 || player[playernum].location == locations[noun].num)
		      {
				tell1("You wave the wand but nothing happens.  That's a pity.");
				continue;
		      }

		      if ((rnd(1,10)) == 1)
		      {
				tell1("You wave the wand.  It tingles a little then goes dead.");
				strcpy(equip[b].name[playernum],"useless wand");
				continue;
		      }
		      strcpy(temp,"You wave the wand.  It tingles in your hand and everything goes black.");
		      strcpy(temp2,player[playernum].name);
		      strcat(temp2," waves a magic wand and suddenly vanishes.");
		      tell1a(temp,temp2);
		      strcpy(temp,"You feel yourself spinning around, then the darkness lifts.");
		      strcpy(temp2,player[playernum].name);
		      strcat(temp2," suddenly appears in a puff of smoke, holding a magic wand.");
		      player[playernum].location = locations[noun].num;
		      tell1a(temp,temp2);
		      break;

	     }
	     if (verb == 13)                     /*  kill   */
	     {
		      if (which == 0)
		      {
			       c = findvictim(player[playernum].location,player[playernum].name);
			       if (c)
			       {
					tell1("You can't find anybody to attack.");
					continue;
			       }
		      }
		      if (which == 2 || which == 8)
		      {
				c = 0;
				d = getattack(8,player,which,noun,temp,temp2);
				a = getattack(which,noun,8,player,temp3,temp4);
				if (which == 2)
				{
					 e = monsproto[mons[noun].type].damage;
					 if (mons[noun].weapon)
						e = (objects[mons[noun].weapon].dd);

					 f = monsproto[mons[noun].type].ds;
					 if (mons[noun].armour)
						f = (objects[mons[noun].armour].dd - 100);

					 c = attack(temp,temp2,d,player[playernum].damage,player[playernum].ds,&player[playernum].life,temp3,temp4,a,e,f,&mons[noun].life);
					 if (c == 1)
					 {
						  monsdropall(noun);
						  player[playernum].xp[1] += mons[noun].skill;
						  strcpy(temp4,"The ");
						  strcat(temp4,monsproto[mons[noun].type].name);
						  askmoney(player,temp4,mons[noun].money,1);

					 }

				}
				else
				{

					c = attack(temp,temp2,d,player[playernum].damage,player[playernum].ds,&player[playernum].life,temp3,temp4,a,player[noun].damage,player[noun].ds,&player[noun].life);
					if (c == 1)
					{
						 player[playernum].xp[1] += player[noun].skills[1];
						 askmoney(player,player[noun].name,player[noun].money,1);
						 askmoney(player,player[noun].name,player[noun].food,2);
					}
					else
					{
						 player[noun].xp[1] += player[playernum].skills[1];
						 askmoney(noun,player[playernum].name,player[playernum].money,1);
						 askmoney(noun,player[playernum].name,player[playernum].food,2);
					}

				}
				if (c == 2)
				{
					dropall();
					quit = 1;
					break;
				}


		      }
		      else
			   no();
		      continue;

	     }
	     if (verb == 14)
	     {
		     if (player[playernum].food < 1)
		     {
			     tell1("You haven't got any food.");
			     continue;
		     }
		     player[playernum].food --;
		     player[playernum].life += 2;
		     tell1("You sit down to eat some delicious dried rations.");
		     continue;
	     }
	     if (verb == 15)
		     pray();







       }  /* Player input loop (second loop) ends here   */




    }  /*  Main loop ends here  */

    if (player[playernum].life < 1)
	  death();
    else
    {
	  strcpy(temp,player[playernum].name);
	  strcat(temp," vanishes in a puff of smoke.");
	  tell1a("You quit, player coward.",temp);
    }
    writeobjects(temp);
    writemons(temp);
    writeplayer();

}   /*   Program ends here   */




/*
Give introduction (Treetop times)

Monsters:
	   maybe attack player

     ?Mail - read, write

     praying - results, xp

     stealing - money - from shopstuff, from monsters.  Check for success, mons
			attacking, decrease shopmoney/mons.money, spirit decrease
	      - objects - from shopstuff. Check for success, change avail status,
			add to equip
     buying from shopstuff - check if available,
	  check if player already have armour/weapon, increase shopmoney,
	  maybe change avail status, add to equip array, weight
     selling to shopstuff - check if it's buyable (ie in shopstuff array),
	  change avail status, decrease shopmoney, delete from your array,
	  check your DS etc, weight

 Commands for players with special privileges:
     Add - item to objects array or shopstuff array
	   weapon
	   armour           name, desc, weight, cost, DS/dam etc
	   monster
	(if there's space in array)
	Resave appropriate arrays
	   location - find a blank location.  Enter name, desc.  Resave array
*/



/*  Functions:      */


void adjust()               /* Find your DS, armour & weapons no's */
{
      int a;
      player[playernum].ds = 0;
      player[playernum].damage = 0;
      player[playernum].armour = 0;
      player[playernum].weapon = 0;
      for (a = 0; a <= player[playernum].carry; a++)
      {
	      if (equip[a].dd[playernum] > 100)
	      {
		     player[playernum].ds = (player[playernum].ds + (equip[a].dd[playernum] - 100));
		     player[playernum].armour = a;
	      }
	      if (equip[a].dd[playernum] > 0 && equip[a].dd[playernum] < 100 && equip[a].dd[playernum] > player[playernum].damage)
	      {
		     player[playernum].damage = equip[a].dd[playernum];
		     player[playernum].weapon = a;
	      }
     }
     if (player[playernum].weapon == 0)
	      player[playernum].damage = (player[playernum].skills[2] + 1);
}


void askmoney(int player,char name[60],int b,int what)
{                                            /* What: 1 = cowrie shells   */
	int a;                               /*       2 = food            */
	char temp [80];                      /* b = amount                */
	char temp2 [30];                     /* Name is name of victim    */
	char temp3 [60];
	if (b)
	{
		 itoa(b,temp2,10);
		 strcpy(temp,name);
		 if (b == 1)
		 {

			if (what == 1)
			{
				strcat(temp," was carrying 1 measly cowrie shell.");
				tell1(temp);

			}
			else
			{
				strcat(temp," had 1 day's worth of rations.");
				tell1(temp);

			}
			strcpy(temp3,"Do player want it?  ");
		 }
		 else
		 {
			strcat(temp," was carrying ");
			strcat(temp,temp2);
			if (what == 1)
				strcat(temp," cowrie shells.");
			else
				strcat(temp," rations of food.");
			tell1(temp);
			strcpy(temp3,"Do player want them?  ");
		 }
		 strcpy(temp,"");
		 strcpy(temp2,"");
		 a = yesorno(temp3);
		 if (a)
		 {
			if (what == 1)
				player[playernum].money += b;
			else
				player[playernum].food += b;
			if (player[playernum].job == 3)
			{
				tell1("Robbing dead bodies does not please the gods!");
				lower(3);
			}


		 }

	}

}


					      /* Continuous fighting   */
int attack(char uattack1 [80],char itattack1 [80],int skill1,int dam1,int ds1,int *life1,char uattack2 [80],char itattack2 [80],int skill2,int dam2,int ds2,int *life2)
{
       int as, dead = 0;                     /* returns 1 if defender dies */
       while (*life1 > 0 && *life2 > 0)      /* 2 if attacker dies         */
       {                                     /* 0 if neither die           */
	       as = (skill1 + (rnd(0,5)) - ds2);
	       if (as > dam1)
		       as = dam1;
	       if (as > 0)
	       {
		      *life2 -= as;

	       }
	       if (life2 > 0)
	       {
		       as = (skill2 + (rnd(0,5)) - ds1);
		       if (as > dam2)
			       as = dam2;
		       if (as > 0)
		       {
			       *life1 -= as;


		       }

	       }

       }
       if (*life1 < 1)
       {
	     puts("attacker dies");
	     dead = 2;
       }
       else
       {
	     puts("defender dies");
	     dead = 1;
       }
	return(dead);
}

void choosenamejob(int bestapp)
{
	   int a;
	   char temp [80], temp2[80];
	   char firstname [30]; secondname [30];

	   if (player[playernum].age == 8 || player[playernum].age == 11 || player[playernum].age == 18)
		sprintf(temp,"You are an %i year old %s %s.  You are %i cm tall and weigh %i kg.",player[playernum].age,player[playernum].sex,species[player[playernum].species],player[playernum].height,player[playernum].weight);
	   else
		sprintf(temp,"You are a %i year old %s %s.  You are %i cm tall and weigh %i kg.",player[playernum].age,player[playernum].sex,species[player[playernum].species],player[playernum].height,player[playernum].weight);
	   tell1(temp);
	   switch (player[playernum].species)
	   {
		    case HUMAN: humandesc ();tell1(player[playernum].desc);humantown ();break;
		    case MONKEY: monkeydesc ();tell1(player[playernum].desc);monkeytown ();break;
	   }
	   tell1("You have just finished your basic education.  It's time to choose a career.");
	   tell1("Your teacher has assessed your apptitude for various careers.");
	   tell1("Career            Apptitude");
	   for (a = 0;a < MAXJOBS; a++)
	   {
		sprintf (temp,"%i  %-15s%s",a+1,jobs[a],player[playernum].apptitude[a]);
		tell1(temp);
	   }
	   sprintf(temp,"Your teacher advises you to become a %s.",jobs[bestscore]);
	   tell1(temp);
	   do
		getinput(temp,temp2,"Which one do you choose?  ");
	   while (atoi(temp) < 1 || atoi(temp) > MAXJOBS);
	   player[playernum].job = atoi(temp);
	   tell1("The custom in this land is that when you change careers, you choose a new");
	   tell1("name for yourself that reflects your new social role.");
	   while (strcmp(player[playernum].name,"") == 0)
	   {
		    getinput(firstname,secondname,"What name will you be known by?  ");
		    strcat(player[playernum].name,firstname);
		    if(strcmp(secondname,"") && (strlen(player[playernum].name) + strlen(secondname)) < MAXNAME)
		    {
			    strcat(player[playernum].name," ");
			    strcat(player[playernum].name,secondname);
		    }
	   }
	   describetraining ();
	   if (player[playernum].species == MONKEY)
		player[playernum].age += 1;
	   if (player[playernum].species == HUMAN)
		player[playernum].age += 3;
}

void humandesc ()
{
	char *skincol [4] =
	{
		"fair",
		"light brown",
		"dark brown",
		"olive",
	};
	char *haircol [6] =
	{
		"fair",
		"red",
		"light brown",
		"brown",
		"dark brown",
		"black"
	};
	char *eyecol [5] =
	{
		"blue",
		"green",
		"grey",
		"brown",
		"black",
	};

	sprintf (player[playernum].desc,"You have %s skin, %s hair and %s eyes.",skincol[rnd(0,3)],haircol[rnd(0,5)],eyecol[rnd(0,4)]);
}

void monkeydesc ()
{
	char *haircol [6] =
	{
		"soft, light green",
		"course, dark green",
		"reddish-brown",
		"brown",
		"golden",
		"black"
	};
	char *eyecol [5] =
	{
		"big",
		"green",
		"brown",
		"little beady",
		"black"
	};

	sprintf (player[playernum].desc,"You have %s fur and %s eyes.",haircol[rnd(0,5)],eyecol[rnd(0,4)]);
}

int checkdd(int a,int x)               /* a = mons number, x = objectsect num.  */
{                                      /* If it's an appropriate armour or  */
       int ok = 1, small = 0;          /* weapon, pick it up & return 0     */
       if(objects[x].dd == 0)              /* else return 1                     */
	      return(1);
       if(monsproto[mons[a].type].aw == 0)
	      return(1);
       if (strstr(objects[x].name,"small") != NULL || strstr(objects[x].name,"short") != NULL)
	      small = 1;
       if( (small == 0 && objects[x].dd < 100 && monsproto[mons[a].type].aw == 1) || (small == 1 && objects[x].dd < 100 && monsproto[mons[a].type].aw == 2) )
       {
    /*          printf("checkdd:  mons no = %i mons aw = %i\n",a,monsproto[mons[a].type].aw);  */
	      objects[x].location = (a + 700);
    /*          printf("objectsect location = %i\n",objects[x].location); */
	      mons[a].weapon = x;
	      ok = 0;
       }
       return(ok);

}



void checknoun(char temp2 [30])
{
	int b = 0;
	noun = 0;
	which = 0;

	if (stricmp(temp2,"all") == 0)
	{
	      which = 6;
	      return;
	}
	for (b = 0; b <= player[playernum].carry;b++)
	{
		if (strnicmp(temp2,equip[b].name[playernum],strlen(temp2)) == 0)
		{
			noun = b;
			which = 3;
			return;
		}
	}

	for (b = 0;b < MAXOBJECTS;b++)
	{
	       if (strnicmp(temp2,objects[b].name,strlen(temp2)) == 0 && objects[b].location == player[playernum].location )
	       {
		      noun = b;
		      which = 1;
		      return;
	       }
	 }
	 for (b = 0; b <= mon;b++)
	 {
	       if (strnicmp(temp2,monsproto[mons[b].type].name,strlen(temp2)) == 0 && mons[b].location == player[playernum].location && mons[b].life > 0)
	       {
		      noun = b;
		      which = 2;
		      return;
	       }
	 }
	 for (b = 0; b <= player;b++)
	 {
	       if (strnicmp(temp2,player[b].name,strlen(temp2)) == 0 && player[b].location == player[playernum].location && b != player)
	       {
		      noun = b;
		      which = 8;
		      return;
	       }
	 }

	 for (b = 1; b < location;b++)
	 {
	       if (stricmp(temp2,locations[b].name) == 0)
	       {
		      noun = b;
		      which = 7;
		      return;
	       }
	 }

	if (player[playernum].location != 20)
	      return;
	for (b = 0; b <= equ;b++)
	{
	       if (strnicmp(temp2,shopstuff[b].name,strlen(temp2)) == 0 && shopstuff[b].avail == 1)
	       {
		      noun = b;
		      which = 4;
		      return;
	       }
	}
	for (b = 0; b <= arm; b++)
	{
	      if (strnicmp(temp2,shopstuff[b].name,strlen(temp2)) == 0 && shopstuff[b].avail == 1)
	       {
		      noun = b;
		      which = 5;
		      return;
	       }
	}

}


void checkverb(char temp [30])
{

	int b = 0;
	verb = 0;
	for (b = 1; b < MAXVERBS;b++)
	{
		if (strnicmp(temp,verbs [b],strlen(temp)) == 0)
		{
			verb = b;
			break;
		}
	}

}


void clearobjectsarray ()
{
	int loop;

	for (loop = 0; loop < MAXOBJECTS; loop++)
	{
	       objects[loop].type = -1;
	       objects[loop].location = -1;
	       objects[loop].owner = -1;
	}
	for (loop = 0; loop < MAXSHOPSTUFF; loop++)
	{
	       strcpy(shopstuff[loop].name,"");
	       strcpy(shopstuff[loop].desc1,"");
	       strcpy(shopstuff[loop].desc2,"");
	       shopstuff[loop].cost = shopstuff[weight] = -1;
	       shopstuff[loop].available = NO;
	       shopstuff[loop].damage = shopstuff[loop].ds = shopstuff[loop].type = -1;
	}
}


void clearmonsarray()
{
	int a;

	for (a = 0;a < MAXMONSPROTO;a++)
	{
		strcpy(monsproto [a].name,"");
		strcpy(monsproto [a].desc,"");
		strcpy (monsproto [a].walk,"");
		strcpy (monsproto [a].attack,"");
		monsproto [a].terrain = -1;
		monsproto [a].life = -1;
		monsproto [a].ds = -1;
		monsproto [a].skill = -1;
		monsproto [a].money = -1;
		monsproto [a].damage = -1;
		monsproto [a].armour = -1;
		monsproto [a].mood = -1;
		monsproto [a].weapon = -1;
	}
	for (a = 0;a < MAXMONS;a++)
	{
		mons [a].type = -1;
		mons [a].life = -1;
		mons [a].location = -1;
		mons [a].mood = -1;
		mons [a].skill = -1;
		mons [a].armour = -1;
		mons [a].weapon = -1;
		mons [a].money = -1;
	}
}


void clearlocarray()
{
	    int a, b;

	    for (a = 0;a < MAXLOCATIONS;a++)
	    {
		  strcpy(locations[a].name,"");
		  locations[a].terrain = -1;
		  locations[a].bridge = -1;
		  for (b = 0;b < MAXLOCATIONDESC;b++)
			  strcpy(locations[a].desc[b],".");
		  for (b = 0; b < MAXDIR; b ++)
			locations[a].map[b] = -1;
	    }
}



void createmons(int monsnum)
{
      int monstype;
      char temp [80];

   /* puts("In createmons");       */
      monstype = rnd(0,MAXMONSPROTO -1);
      mons[monsnum].type = monstype;
      mons[monsnum].life = monsproto[monstype].life + (rnd(0,20)) - 10;
      if (mons[monsnum].life < 1)
		mons[monsnum].life = 0;
      mons[monsnum].skill = monsproto[monstype].skill + (rnd(0,10)) - 5;
      if (mons[monsnum].skill < 1)
		mons[monsnum].skill = 0;
      mons[monsnum].money = (rnd(1,monsproto[monstype].money));
      mons[monsnum].mood = monsproto[monstype].mood;
      assignhome (monsproto[monstype].terrain,&(mons[monsnum].location));
      sprintf (temp,"A %s %s over.",monsproto[monstype].name,monsproto[monstype].walk);
      tellallpresent (temp,mons[monsnum].location);
      mons[monsnum].armournum = assignobject(monsproto[monstype].armour,monsnum,MONSTER);
      mons[monsnum].weaponnum = assignobject(monsproto[monstype].weapon,monsnum,MONSTER);
}

void createchr()
{
/* Create a new character (except for char name, desc, job) */
/* player[playernum].player is assumed to be set already */
	   int a, bestapp;

	   player[playernum].species = (rnd(0,MAXSPECIES));
	   if (player[playernum].species == HUMAN)
	   {
		player[playernum].age = (rnd(18,60));
		player[playernum].height = (rnd(150,210));
		player[playernum].weight = (rnd(40,150));
	   }
	   if (player[playernum].species == MONKEY)
	   {
		player[playernum].age = (rnd(1,15));
		player[playernum].height = (rnd(50,110));
		player[playernum].weight = (rnd(20,75));
	   }
	   switch ((rnd(0,2))
	   {
		case 0:strcpy(player[playernum].sex,"female");break;
		case 1:strcpy(player[playernum].sex,"neuter");break;
		case 2:strcpy(player[playernum].sex,"male");break;
	   }
	   bestapp = 0;
	   for(a = 0;a < MAXJOBS;a++)
	   {
		  player[playernum].apptitude[a] = (rnd(0,2));
		  if (player[playernum].apptitude[a] >= player[playernum].apptitude[bestapp])
			bestapp = a;
	   }
	   choosenamejob(bestapp);
	   player[playernum].money = (rnd(1,10));
	   player[playernum].ds = 1;
	   player[playernum].damage = 1;
	   player[playernum].location = STARTLOC;
	   player[playernum].life = rnd(10,30);
	   player[playernum].weapon = -1;
	   player[playernum].armour = -1;
	   player[playernum].weight = 0;
	   player[playernum].hunger = 0;
	   player[playernum].energy = player[playernum].life / 2;
	   player[playernum].relativeloc = STARTLOC;
	   for (a = 0; a < MAXJOBS; a ++)
	   {
		  player[playernum].xp[a] = 0;
		  player[playernum].skills[a] = 0;
	   }
	   player[playernum].skills[player[playernum].job] = 1;
	   switch (player[playernum].job)
	   {
		case MAGICIAN:
	   }
}

void death()
{
      tell1("You have died.");
}


void dropall()
{
	char temp [80];
	char temp2 [80];
	char temp3 [80];
	int a, b = 0, c;
	for (a = 1;a <= player[playernum].carry;a++)
	{
		  strcpy(temp,equip[a].name[playernum]);
		  c = dropit(a);
		  if (c < 2)
		  {
			   strcpy(temp2,"You drop the ");
			   strcat(temp2,temp);
			   strcat(temp2,".");
			   strcpy(temp3,player[playernum].name);
			   strcat(temp3," drops a ");
			   strcat(temp3,temp);
			   strcat(temp3,".");
			   tell1a(temp,temp3);
			   if (c == 1)
				     hermes(temp);
		  }

	}
	sortequip();
	adjust();
}



int dropit (int a)                           /*  0 = success               */
{                                            /*  1 = success but objectsect    */
    int c = 2, loop;                         /*      disappears            */
					     /*  2 = error                 */
					     /*  3 = don't drop valuables  */

    if (stricmp(equip[a].name[playernum],"lunch box") == 0)
	    return(3);
    if (stricmp(equip[a].name[playernum],"spell book") == 0)
	    return(3);
    if (strcmp(equip[a].name[playernum],"-"))
    {
	   c = 1;
	   for (loop = 1; loop < MAXOBJECTS; loop ++)
	   {
		  if (strcmp(objects[loop].name,"-") == 0)
		  {
			 c = 0;
			 strcpy(objects [loop].name,equip[a].name[playernum]);
			 objects [loop].weight = equip[a].weight[playernum];
			 objects [loop].location = (int) player[playernum].location;
			 objects [loop].dd = equip[a].dd[playernum];
			 break;
		  }
	   }
	   player[playernum].weight = player[playernum].weight - equip[a].weight[playernum];
	   strcpy(equip[a].name[playernum],"-");
	   equip[a].weight[playernum] = 0;
	   equip[a].dd[playernum] = 0;

    }
return(c);
}


int findvictim(float atlocation, char atname [30])
{                                        /* Find a mons/person to attack  */
					 /* 0 = success, 1 = failure      */
	int a, b, c = 0;                 /* ATNAME = name of attacker     */
					 /* ATlocation = location no of attacker  */
	noun = 9999;                     /* NOUN & WHICH are set to attackee */
	for (a = 0;a <= mon; a++)
	{
		   if (mons[a].location == atlocation && strcmp(monsproto[mons[a].type].name,atname) != 0)
		   {
			     noun = a;
			     which = 2;
			     c = 0;
			     break;

		   }
	}
	if (noun == 9999)
	{
		   for (a = 0; a <= player; a++)
		   {
			     if (player[a].location == player[playernum].location && strcmp(player[a].name,atname) != 0)
			     {
				      noun = a;
				      which = 8;
				      c = 0;
				      break;
			     }
		    }

       }
       if (noun == 9999)
		 c = 1;
return(c);
}

int getattack(int what,int num,int defwhat,int defnum,char uattack [80],char itattack [80])
{                                         /* Defwhat is defender's type     */
	  char defname [30];              /* Defnum is defender's array num */
	  char temp [80];                 /* What = type of attacker:       */
	  char temp2 [80];                /*    2 (mons) or 8 (person)      */
	  int as;                         /* num = attacker's array number  */
					  /* Returns 2 strings to describe  */
					  /*  the attack from 2 points of view*/
					  /* and the attacker's A.S.        */
	  if (defwhat == 2)
	  {
		 strcpy(defname,"the ");
		 strcat(defname,monsproto[mons[defnum].type].name);
	  }
	  else
	  {
		 strcpy(defname,player[defnum].name);
	  }
	  if (what == 2)
	  {
		 as = mons[num].skill;
		 strcpy(itattack,"The ");
		 strcat(itattack,monsproto[mons[num].type].name);
		 strcat(itattack," ");
		 strcat(itattack,monsproto[mons[num].type].attack);
	  }
	  else
	  {
		 strcpy(uattack,"You");
		 strcpy(itattack,player[num].name);
		 strcat(itattack," ");

		 if (player[num].weapon)
		 {
			as = player[num].skills[1];
			strcat(uattack," hit ");
			strcpy(uattack,defname);
			strcpy(uattack,".");
			strcat(itattack," hits player.");

		 }
		 else
		 {
			as = player[num].skills[2];
			getma(player[num].skills[2],temp,temp2);
			strcat(uattack,temp);
			strcat(uattack,defname);
			strcat(uattack,".");
			strcat(itattack,temp2);
			strcat(itattack,"player.");
		 }

	  }
	  return(as);
}


int getinput(char word1 [],char word2[],char promptstr[])
{
/* Accepts 1 or 2 words of input and returns them in word1 and word2. */
     char inputline[30];                               /* Returns no. of words */
     int a,b,c,d,e;                                /* input (1 or 2)       */
     strcpy(word1,"");
     strcpy(word2,"");

     while (strcmp(word1,"") == 0)
     {
	   strcpy(inputline,"");
	   b = c = d = e = 0;
	   while (strcmp(inputline,"") == 0)
	   {
	       prompt(promptstr);
	       fflush(stdin);
	       gets(inputline);
	   }


	   for (a = 0; a <= strlen(inputline);a++)
	   {
	       if (inputline[a] == '\0')
		     break;
	       if (inputline[a] == ' ')
	       {
		     if (b)
			  break;
		     if (strcmp(word1,"") == 0)
			  continue;
		     b = 1;
		     continue;
		}
		if (b == 0)
		{
		     word1[d] = inputline[a];
		     d += 1;
		}
		else
		{
		     word2[e] = inputline[a];
		     e += 1;
		}
	   }

     }

     word1[d] = '\0';
     word2[e] = '\0';
     c = 1;
     if (strcmp(word2,""))
	    c += 1;
     return(c);
}

						/* Returns:               */
int getit(int a)                                /* 0 = success            */
{                                               /* 1 = carrying too much  */
      int c;
      if (objects[a].location != player[playernum].location)         /* 2 = not carrying it    */
	   return(2);
      if ( player[playernum].weight + objects[a].weight < MAXWEIGHT)
      {
	   c = 0;
	   player[playernum].weight = player[playernum].weight + objects[a].weight;
	   player[playernum].carry += 1;
	   strcpy(equip[player[playernum].carry].name[playernum],objects[a].name);
	   equip[player[playernum].carry].dd[playernum] = objects [a].dd;
	   strcpy(objects[a].name,"-");
	   strcpy(objects[a].desc,"-");
	   objects[a].weight = 0;
	   objects[a].location = 0;
	   objects[a].dd = 0;
      }
      else
	   c = 1;

return(c);
}


void getline(char temp[80], char temp4[80])      /* Input (local or remote) */
{
     prompt(temp4);
     fflush(stdin);
     while (strcmp(temp,"") == 0)
	    gets(temp);
}

void getma(int skill,char uattack [80],char itattack [80])
{                                               /*  Takes your skill and  */
      int a;                                    /*  returns a rnd attack   */
      a = (rnd(1,skill));
      if (a < 2)
      {
	     strcpy(uattack," punch ");
	     strcpy(itattack," punches ");
	     return;
      }
      if (a == 2)
      {
	     strcpy(uattack," kick ");
	     strcpy(itattack," kicks ");
	     return;
      }
      if (a == 3)
      {
	      strcpy(uattack," elbow strike ");
	      strcpy(itattack," elbow strikes ");
	      return;
      }
      strcpy(uattack," throw ");
      strcpy(itattack," throws ");


}


float getrelative(float a)                 /*  Takes a real location no and */
{                                          /*  and returns the fake location no */
       float b = 0;
       int c = MAXLOCATIONS -1;
       if ( (int) (a / c) == (a / c) )
	     b = 15;
       else
	     b = (a - (( (int) (a / c) ) * c) );

       return (b);

}


void help()
{
	int a;
	tell1("There are the commands player can use (the first letter will do):");
	for (a = 1;a < MAXVERBS;a++)
	      tell1(verbs[a]);

}


void hermes(char temp[MAXNAME])
{
     char temp2 [80];
     tellallpresent("Hermes, messanger of the gods, appears out of nowhere.",player[playernum].location);
     strcpy(temp2,"He picks up the ");
     strcat(temp2,temp);
     strcat(temp2," and disappears.");
     tellallpresent(temp2,player[playernum].location);
}

void describetraining()
{
/* Briefly describe the player's early training in a career and their */
/* reason for going off on an adventure. */

	 if (player[playernum].species == HUMAN)
		switch (player[playernum].job)
		{
			case MAGICIAN:tell1("The practise of magic is strongly discouraged in your society");
			tell1("so you secretly take up study under an old mage named Shurnuth.");
			tell1("He teaches you much about the history, working and uses of Magic.");
			tell1("He gives you an old reference book containing instructions and spells.");
			tell1("He tells you, 'Never attempt to cast a spell that you don't fully understand.");
			tell1("The results could be disasterous!'");
			tell1("You spend three years studying basic magical skills with Shurnuth");
			tell1("until one day you are caught and are run out of town.");
			assignobject(SPELLBOOK,playernum,PLAYER);
			break;

			case WARRIOR:tell1("You join the Hihun Warrior School, seeking to become a brave and");
			tell1("fearless fighter.");
			tell1("You train with all kinds of weapons for a few years.");
			tell1("However, to become a true warrior you must venture out into the");
			tell1("wilderness alone and seek your destiny.");
			assignobject(HEAVYWEAPON,playernum,PLAYER);
			break;

			case MONK:tell1("You feel that your true destiny lies in serving the gods.");
			tell1("You leave your home and all your earthly posessions to travel to the");
			tell1("Quecheb monastary, which lies many miles away.");
			tell1("There you spend many hours every day in prayer and meditation.");
			tell1("Your prayer book is your only valuable posession.");
			tell1("After a few years you decide to leave to spread religion throughout");
			tell1("the land.");
			assignobject(PRAYERBOOK,playernum,PLAYER);
			break;

			case THIEF:tell1("Your natural mischeviousness and complete lack of morals lead you to");
			tell1("become a thief.");
			tell1("You read every scroll on lock-picking that you can find, and practise");
			tell1("picking pockets and shoplifting to survive.");break;
			tell1("Unfortunately, after a few years of this you are caught stealing food");
			tell1("from a shop and are forced to leave town in a hurry to avoid");
			tell1("imprisonment.  You don't have time to take much with you.");
			assignobject(
			break;

			case TRADER:tell1("Being greedy, you devote your life to making as much money");
			tell1("as possible through trading.  You ask other traders in the town to teach");
			tell1("you their sales secrets.");
			tell1("You like travelling and adventure, so after a few years you leave to");
			tell1("explore the land.");
			break;
		}
	 if (player[playernum].species == MONKEY)
		switch (player[playernum].job)
		{
			case MAGICIAN:tell1("You ask the oldest, wisest monkeys to teach you magic.");
			tell1("You record your knowledge in a spell book.");
			tell1("However, the monkeys have only a limited knowledge of magic so after");
			tell1("a year of study you leave to go travelling.");
			break;

			case WARRIOR:tell1("You join the monkey soldiers who defend the territory");
			tell1("and learn to use all kinds of weapons.");
			tell1("After a year you leave them to venture out into the land to prove");
			tell1("yourself.");
			break;

			case MONK:tell1("You decide to devote your life to serving the wishes of the spirits who");
			tell1("created this land.");
			tell1("You spend a year with the other monkey priests.  However, you feel");
			tell1("that the spirits are calling you out into the wildernesss, so you");
			tell1("leave the community and venture out alone.");
			break;

			case THIEF:tell1("Being a mischevious monkey, you decide you would rather steal than");
			tell1("work for a living.");
			tell1("A year or so later you are caught stealing fruit from an elderly");
			tell1("monkey and are thrown out of the community in disgrace.");
			break;

			case TRADER:tell1("You decide to make as much money as possible from buying and");
			tell1("selling anything and everything.");
			tell1("There are very limited opportunities in your community, so you");
			tell1("leave to find a better place to work.");
			break;
		}
}

void humantown ()
{
/* Briefly describe the character's background. */
	  char temp [80];
	  switch(rnd(0,6))
	  {
		  case 0:tell1("You grew up in Wovoja village, on the edge of the Lucol desert.");break;
		  case 1:tell1("You grew up in the small town of Viniv, in the Bohosh Duchy area.");break;
		  case 2:tell1("You have lived on a farm on Vevos island since birth.");break;
		  case 3:tell1("You grew up in the fishing village of Noniz.");break;
		  case 4:tell1("You were raised in Zimuch city, in the Kudoz Valley.");break;
		  case 5:tell1("You have been living with the nomadic Ehip Kukido tribe.");break;
		  case 6:tell1("You have been living in a small community in the Ocheztedo rainforest.");break;
	  };
	  switch(rnd(0,9))
	  {
		case 0:tell1("Your family are beggers.");break;
		case 1:tell1("Your mother is the local witch doctor.");break;
		case 2:tell1("You are an orphan.");break;
		case 3:tell1("You were raised by your grandparents.");break;
		case 4:tell1("Your parents are jugglers and fortune tellers.");break;
		case 5:tell1("Your parents are poitical activists.");break;
		case 6:tell1("You were raised by a group of satanic priests.");break;
		case 7:tell1("Your mother is an artist and your father is an animal trainer.");break;
		case 8:tell1("Life is very hard there.");break;
		case 9:tell1("Life is very boring there.");break;
	  };
}

void monkeytown ()
{
	switch(rnd(0,7))
	{
		case 0:tell1("You live with your family in the jungle, far from any civilisation.");break;
		case 1:tell1("Your family lived a zoo in Zimuch city, but escaped to the jungle.");break;
		case 2:tell1("You have been living in a small monkey community in the Ocheztedo forest.");break;
		case 3:tell1("You were brought up by humans in the Wovoja village.");break;
		case 4:tell1("You live with relatives in scrubland in the Kudoz Valley.";break;
		case 5:tell1("You live with 15 siblings in the depths of the Fabdevi forest.";break;
		case 6:tell1("You roam with your family on the grasslands outside of the Noniz village.");breal;
		case 7:tell1("You live with a monkey tribe in the haunted Ezaaw forest.");break;
	};
}

void inventory()
{
	  int a;
	  char temp [50];
	  char temp2 [80];
	  tell1("You are carrying:");
	  for (a = 1; a <= player[playernum].carry; a++)
		   tell1(equip[a].name[playernum]);
	  if (player[playernum].weapon)
		   strcpy(temp,equip[player[playernum].weapon].name[playernum]);
	  else
		   strcpy(temp,"none");
	  strcpy(temp2,"Your weapon:  ");
	  strcat(temp2,temp);
	  tell1(temp2);
	  itoa(player[playernum].money,temp,10);
	  strcpy(temp2,"Cowrie shells:  ");
	  strcat(temp2,temp);
	  tell1(temp2);
	  itoa(player[playernum].food,temp,10);
	  strcpy(temp2,"No. of days' food:  ");
	  strcat(temp2,temp);
	  tell1(temp2);

}



void lower(int score)                           /* Lose a skill level */
{                                               /* Holiness can go negative */

	if (player[playernum].skills[score] > 0 || score == 3)
		player[playernum].skills[score] =- 1;
	else
		player[playernum].xp[score] = 0;
}


void monsdrop(int num, int what)
{
       char temp4 [80];
       if (objects[what].location == (num + 700))
       {
		strcpy(temp4,"The ");
		strcat(temp4,monsproto[mons[num].type].name);
		strcat(temp4," drops a ");
		strcat(temp4,objects[what].name);
		strcat(temp4,".");
		tellallpresent(temp4,objects[what].location);
		objects[what].location = mons[num].location;
       }
}


void monsdropall(int num)
{
	int a;
	for (a = 0; a < MAXOBJECTS; a++)
		 monsdrop(num,a);

}


void monsmove(int a)                             /* a = monster number  */
{
       int b = 0, c = 0, done = 0, drop = 0;
       char temp4 [80];
       if (mons[a].life < 1)
       {
		for (b = 0; b <= player; b++)
		{
			if (player[playernum].location == b)
				c = 1;
		}
		if (c == 1 || ((rnd(1,3)) != 1) )
			return;
		else
		{
			/*     puts("resurrection in progress");    */
			createmons(a);
			return;
		}

       }
       if (monsproto[mons[a].type].money != 0 || monsproto[mons[a].type].aw != 0)
       {
	      for (b = 1; b < MAXOBJECTS; b++)
	      {

		      drop = 0;
		      if(objects[b].location == (700 + a) && ((rnd(1,3)) == 1) && objects[b].dd == 0)
		      {
			      monsdrop(a,b);
			      drop = 1;
			      done = 1;
		      }
		      if (objects[b].location == mons[a].location && drop == 0 && ((rnd(1,3)) == 1))
		      {
			      if (objects[b].dd)
				      c = checkdd(a,b);
			      else
			      {
				      c = 1;
				      if (monsproto[mons[a].type].money)
						c = 0;
			      }

			      if (c == 0)
			      {
				      strcpy(temp4,"The ");
				      strcat(temp4,monsproto[mons[a].type].name);
				      strcat(temp4," picks up the ");
				      strcat(temp4,objects[b].name);
				      strcat(temp4,".");
				      tellallpresent(temp4,objects[b].location);
				      done = 1;
				      if(objects[b].dd == 0)
					      objects[b].location = (700 + a);
			      }
		      }

	      }

       }
       if (done)
	      return;
       b = (rnd(1,4));
       c = map[mons[a].location] [b];
       if (c > 0 && getblock( (float) c) == getblock( (float) mons[a].location) && (locations[c].type == locations[mons[a].location].type || locations[mons[a].location].type == 4 || locations[c].type == 4))
       {

			 done = 1;
			 strcpy(temp4,"The ");
			 strcat(temp4,monsproto[mons[a].type].name);
			 strcat(temp4," ");
			 strcat(temp4,monsproto[mons[a].type].walk);
			 strcat(temp4," away ");
			 strcat(temp4,verbs[b]);
			 strcat(temp4,".");
			 tellallpresent(temp4,mons[a].location);
			 mons[a].location = map[c][b];

			 strcpy(temp4,"A ");
			 strcat(temp4,monsproto[mons[a].type].name);
			 strcat(temp4," ");
			 strcat(temp4,monsproto[mons[a].type].walk);
			 strcat(temp4," in.");
			 tellallpresent(temp4,mons[a].location);
       }


}

int monsweapon(int monster)
{
      int loop, number = 0;
      for (loop = 1; loop < MAXOBJECTS; loop ++)
      {
	       if (objects[loop].location == (700 + monster) && objects[loop].dd > 0 && objects[loop].dd < 100)
	       {
			 number = loop;
			 break;
	       }
      }
      return(number);
}


void no()
{
     if (rnd(1,2) == 1)
	   tell1("That's impossible, my friend.");
     else
	   tell1("That be not possible.");

}


void ok()
{
   tell1("OK");
}

void pray()
{
	int b, job;
	int a = player[playernum].skills[3];
	job = player[playernum].job;
	if (player[playernum].species == 1)
		tell1("You pray to the Spirits of the Dark.");
	if (player[playernum].species == 0)
	{
		if (player[playernum].life < ((player[playernum].scores[3] * 2) / 4) )
			tell1("Being so badly wounded, player pray to Ixeqrisha, God of Healing.");
		else
		{
			if (job == 1)
				tell1("You pray to Alolluwa, the Goddess of War.");
			if (job == 3 || job == 0 || job == 2)
				tell1("You pray to Ofegdifo, the God of Wisdom.");
			if (job == 4)
				tell1("You pray to Ikushfupa, the God of the Winds.");
			if (job == 5)
				tell1("You pray to Epohdodu, the Goddess of Light.");
		}

	}
	raise(3);
	b = 30;
	if (player[playernum].skills[3] > 30)
		b = player[playernum].skills[3] + 10;
	if ( (rnd(1,(b - player[playernum].skills[3]))) == 1)
	{

		if (player[playernum].life < (player[playernum].scores[3] * 2))
		{
			player[playernum].life += (rnd(1,10));
			if (player[playernum].life > (player[playernum].scores[3] * 2))
				player[playernum].life = (player[playernum].scores[3] * 2);
			tell1("You feel stronger now.");
			return;
		}
	}
	if (player[playernum].skills[3] > a)
		tell1("You feel a spiritual awakening.");
	else
		tell1("Your soul is strengthened and player feel better able to face the world.");

}

void prompt(char temp3[40])
{
	  printf("%s",temp3);
}

void raise(int num)
{
	int a;
	a = (rnd(1,5));
	player[playernum].xp[num] += a;
	if (player[playernum].xp[num] >= 40)
	{
		player[playernum].xp[num] -= 40;
		player[playernum].skills[num] += 1;
	}
}

void readextra()
{
     FILE *fp;
     fp = fopen("extra.dat","r");
     if (fp == NULL)
	      exit(1);
     rewind(fp);
     fscanf(fp,"%i",&shopmoney);

     fclose(fp);
}


void readmons()
{
/* Read monster prototypes and also individual monsters, if any. */
/* sudmast.dat and sudmons.dat */
    FILE *fp;
    int a;

    fp = fopen ("sudmast.dat","r");
    if (fp == NULL)
    {
	   puts ("Unable to find sudmast.dat in the current directory.  Aborting.");
	   exit(1);
    }
    master = 0;
    for (a = 1;a < MAXMONSPROTO;a++)
    {
	   if (fscanf(fp,"%[^\n]%*c",monsproto [a].name) < 1)
		break;
      /*   printf("Monster %i %s\n",a,monsproto[a].name);  */
	   fscanf (fp,"%[^\n]%*c",monsproto [a].desc);
	   fscanf (fp,"%[^\n]%*c%[^\n]%*c",monsproto [a].attack,monsproto [a].walk);
      /*   printf("desc: %s\n attack: %s\n walk %s\n",monsproto[a].desc,monsproto[a].attack,monsproto[a].walk);*/
	   fscanf (fp,"%i %i %i %i %i %i %i %i %i%*c",&monsproto [a].terrain, &monsproto [a].life,&monsproto[a].ds,&monsproto[a].skill,&monsproto[a].money,&monsproto[a].damage,&monsproto[a].armour,&monsproto[a].mood,&monsproto[a].weapon);
      /*   printf("terrain: %i life: %i ds: %i skill: %i money: %i damage: %i armour %i mood %i weapon %i\n",monsproto[a].location,monsproto[a].life,monsproto[a].ds,monsproto[a].skill,monsproto[a].money,monsproto[a].damage,monsproto[a].armour,monsproto[a].mood,monsproto[a].weapon);*/
    }
    fclose(fp);
    nummons = -1;
    a = 0;
    fp = fopen("sudmons.dat","rt");                           /* Then individual array */
    if (fp != NULL)
    {
	     while (a < (MAXMONS - 1))
	     {
		   if (fscanf(fp,"%i",&mons[a].type) < 1)
			break;
	       /*  printf("read mons %i type = %i\n",a,mons[nummons].type);  */
		   fscanf(fp,"%i %i %i %i %i %i %i\n",&mons[a].life,&mons[a].location,&mons[a].mood,&mons[a].skill,&mons[a].armournum,&mons[a].weaponnum,&mons[a].money);
	      /*   printf("location %i life %i armour %i skill %i money %i mood %i weapon %i\n",mons[a].location,mons[a].life,mons[a].armour,mons[a].skill,mons[a].money,mons[a].mood,mons[a].weapon); */
		   a ++;
	     }
	     nummons = a - 1;
    }
    fclose(fp);
    else
    {
	     for (a = 0;a < MAXMONS;a++)
		  createmons(a);
	     nummons = a;
    }
}


void readobjects()
{
/* Read the master objects (shopstuff) in sudshop.dat and, if it exists, */
/* active objects in sudobj.dat */
/* The data for the shopstuff[].available is in sudavail.dat */

    FILE *fp;
    int a;

    fp = fopen("sudshop.dat","rt");
    if (fp == NULL)
    {
	 puts ("Unable to find sudshop.dat in the current directory.  Aborting.");
	 exit(1);
    }
    for (a = 0;a < MAXSHOPSTUFF;a++)
    {
	  if (fscanf(fp,"%[^\n]%*c",shopstuff[a].name) < 1)
		break;
	  fscanf(fp,"%[^\n]%*c%[^\n]%*c",shopstuff[a].desc1,shopstuff[a].desc2);
     /*    printf("object %i: name %s\ndesc1 %s\n desc2 %s\n",a,shopstuff[a].name,shopstuff[a].desc1,shopstuff[a].desc2); */
	  fscanf(fp,"%i %i %i %i %i%*c",&shopstuff[a].cost,&shopstuff[a].weight,&shopstuff[a].damage,&shopstuff[a].ds,&shopstuff[a].type);
	  shopstuff[a].available = 0;
     /*   printf("cost %i weight %i  damage %i ds %i type %i\n",shopstuff[a].cost,shopstuff[a].weight,shopstuff[a].damage,shopstuff[a].ds,shopstuff[a].type);   */
     }
     fclose(fp);
     if (fp = fopen ("sudavail.dat","rt"))
	for a = 0; a < MAXSHOPSTUFF; a ++)
		fscanf (fp,"%i\n",&shopstuff[a].available;
     fclose (fp);
     numobjects = -1;
     a = 0;
     if ((fp = fopen("sudobj.dat","rt"))
     {
		while (a < (MAXOBJECTS - 1))
		{
			if (fscanf(fp,"%i %i %i%*c",&objects[a].type,&objects[a].location,&objects[a].owner) < 1)
				break;
			a ++;
		}
		numobjects = a - 1;
     }
     fclose (fp);
}


int readchr()
{
/* player[playernum].playername must already be set to the name to look for */
/* If sudchr.dat exists, read it until the player's record is found or */
/* the end of the file is reached.  Return TRUE if successful, else FALSE */
     FILE *fp;
     char temp [81];
     int foundit = FALSE;

     fp = fopen("sudchr.dat","r");
     if (fp)
     {
	do
	{
	    strcpy(temp,"");
	    if (fscanf(fp,"%[^\n]%*c",temp) < 1);      /* A player's name  */
		break;
	    fscanf(fp,"%[^\n]%*c%[^\n]%*c",player[playernum].name,player[playernum].desc);
	    fscanf(fp,"%i\n%s\n%i\n%i\n%i\n%i\n%i\n%i\n",&player[playernum].job,player[playernum].sex,&player[playernum].money,&player[playernum].ds,&player[playernum].damage,&player[playernum].location,&player[playernum].life,&player[playernum].species);
	    fscanf(fp,"%i\n%i\n%i\n",&player[playernum].weapon,&player[playernum].armour,&player[playernum].weight);
	    fscanf(fp,"%i\n%i\n",&player[playernum].hunger,&player[playernum].energy);
	    for (a = 0; a < MAXJOBS; a++)
		 fscanf(fp,"%i\n",&player[playernum].xp[a]);
	    for (a = 0;a < MAXJOBS;a++)
		fscanf(fp,"%i\n",&player[playernum].apptitude[a]);
	    for (a = 0;a < MAXJOBS;a++)
		 fscanf(fp,"%i\n",&player[playernum].skills[a]);
	    fscanf(fp,"%i\n%i\n",&player[playernum].port,&player[playernum].age);
	    fscanf(fp,"%i\n%i\n",&player[playernum].height,&player[playernum].weight);
	  } while (strcmp(player[playernum].playername,temp));
	  foundit = TRUE;
      }
      fclose(fp);
      return (foundit);
}


void readlocation(char temp [30])
{
    FILE *fp;
    int a = 0, b = 0, c  = 0;
    clearlocarray();
   /*     printf("Reading %s\n",temp);  */
    fp = fopen(temp,"r");
    if (fp == NULL)
	 exit(1);
    rewind(fp);
    for (a=1;a<MAXLOCATIONS;a++)
    {
	  fscanf(fp,"%s",locations[a].name);
	  if (strcmp(locations[a].name,"") == 0)
	       break;
   /*       printf("location:  %s\n",locations[a].name);    */
	  getc(fp);
	  fscanf(fp,"%i %i %i",&locations[a].type,&locations[a].bridge,&locations[a].num);
	  getc(fp);
   /*       printf("%i %i %i\n",locations[a].type,locations[a].bridge,locations[a].num);  */
	  fscanf(fp,"%i %i %i %i",&map[a][1],&map[a][2],&map[a][3],&map[a][4]);
	  getc(fp);
    /*      printf("Map N %i S %i E %i W %i\n",map[a][1],map[a][2],map[a][3],map[a][4]); */
	  for (b=0;b<MAXLOCATIONDESC;b++)
	  {
		fscanf(fp,"%[^\n]",locations[a].desc[b]);
	 /*       printf("desc %i: %s\n",b,locations[a].desc[b]);   */
		getc(fp);
	  }
	  location = a;
     }
     fclose(fp);

}


void readshop()
{
    FILE *fp;
    int a = 0, b =0 , c = 0;
    arm = equ = 0;
    fp = fopen ("shopstuff.dat","r");
    if (fp == NULL)
	    exit(1);
    rewind(fp);
    puts("Reading shopstuff equip");
    for (a = 0;a < MAXSHOPSTUFFe;a++)
    {
	  fscanf (fp,"%[^\n]",shopstuff[a].name);
	  getc(fp);
	  if (strcmp(shopstuff[a].name,"***") == 0)
	  {
		 break;
		 strcpy(shopstuff[a].name,"");
	  }
	  /*      printf("Name %s\n",shopstuff[a].name);    */
	  fscanf(fp,"%[^\n]",shopstuff[a].desc);
	  getc(fp);
	  fscanf(fp,"%[^\n]",shopstuff[a].desc2);
	  getc(fp);
    /*      printf("desc %s\n desc2 %s\n",shopstuff[a].desc,shopstuff[a].desc2);    */
	  fscanf(fp,"%i %i %i %i",&shopstuff[a].cost,&shopstuff[a].weight,&shopstuff[a].dd,&shopstuff[a].avail);
	  getc(fp);
     /*     printf("cost %i weight %i avail %i dd %i\n",shopstuff[a].cost,shopstuff[a].weight,&shopstuff[a].dd,shopstuff[a].avail);     */
	  if ( (rnd(0,5)) == 1)
		 shopstuff[a].avail = 1 - shopstuff[a].avail;
    }
    equ = a - 1;
    for (a=0;a < MAXSHOPSTUFF;a++)
    {
	  fscanf (fp,"%[^\n]",shopstuff[a].name);
	  getc(fp);
	  if (strcmp(shopstuff[a].name,"***") == 0 && arm != 0)
	  {
		strcpy(shopstuff[a].name,"-");
		break;
	  }
	  if (strcmp(shopstuff[a].name,"***") == 0)
	  {
		 arm = a - 1;
		 fscanf (fp,"%[^\n]",shopstuff[a].name);
		 getc(fp);
	  }
	  fscanf(fp,"%[^\n]",shopstuff[a].desc);
	  getc(fp);
     /*     printf("Name: %s\nDesc: %s\n",shopstuff[a].name,shopstuff[a].desc);      */
	  fscanf(fp,"%i %i %i %i",&shopstuff[a].cost,&shopstuff[a].weight,&shopstuff[a].dd,&shopstuff[a].avail);
	  getc(fp);
     /*     printf("cost: %i  weight: %i  dd: %i  avail: %i\n",shopstuff[a].cost,shopstuff[a].weight,shopstuff[a].dd,shopstuff[a].avail);      */
	  if (rnd(0,5) == 1)
	       shopstuff[a].avail = 1 - shopstuff[a].avail;

    }
    if (arm == 0)
	arm = MAXSHOPSTUFF;
    fclose(fp);
}


void say (char temp[80], int num)             /* num = player number   */
{
	     int where;
	     where = player[num].port;         /* 0 = local, 999 = put on disk */
					    /* 1000 = computer chr  */
	     if (where == 0)
	     {
			printf("%s\n",temp);       /* Local  */
			return;
	     }
	     if (where == 999)
	     {                                  /* Put on disk  */
			return;
	     }
	     if (where == 1000)                   /* Do nothing */
			return;

	    /*  Put out through com WHERE  */

}



void score()
{
	char skillname [MAXJOBS] [13] =
	{
		"Magic",
		"Fighting",
		"Holiness",
		"Thieving",
		"Trading",
	};
	char playerdesc [80];

	sprintf(playerdesc,"Your name is %s.",player[playernum].name);
	tell1(playerdesc);
	sprintf(playerdesc,"You are a %s %s.",player[playernum].sex,species[player[playernum].species]);
	tell1(playerdesc);
	sprintf(playerdesc,"You're a %s.",jobs[player[playernum].job]);
	tell1(playerdesc);
	tell1("Your characteristics:");
	sprintf(playerdesc,"Life points:   %i.",player[playernum].life);
	tell1(playerdesc);
	tell1("Your skill levels:");
	for (a = 0; a < MAXJOBS; a++)
	{
		sprintf(playerdesc,"%s:  %15i",skillname[a],player[playernum].skills[a]);
		tell1(playerdesc);
	}
}

void sortequip()
{
      int a;

      player[playernum].carry = 0;
      for (a = 1; a < (maxequip - 1);  a++)
      {
	   if (strcmp(equip[a].name[playernum],"-") == 0 && strcmp(equip[a+1].name[playernum],"-") != 0)
	   {
		 if (player[playernum].weapon == (a + 1))
			player[playernum].weapon = a;
		 if (player[playernum].armour == (a + 1))
			player[playernum].armour = a;

		 strcpy(equip[a].name[playernum],equip[a+1].name[playernum]);
		 equip[a].weight[playernum] = equip[a+1].weight[playernum];
		 equip[a].dd[playernum] = equip[a+1].dd[playernum];

		 strcpy(equip[a+1].name[playernum],"-");
		 equip[a+1].weight[playernum] = 0;
		 equip[a+1].dd[playernum] = 0;
	   }

      }
      for (a = 1; a < maxequip; a++)
      {
		if (strcmp(equip[a].name[playernum],"-") )
			player[playernum].carry = a;
      }
}




void tell(char msg[85], char someone[85], int num2, char others[85])
{                                      /* MSG = msg to the initiator.  */
				       /* SOMEONE = msg to reveiver (num2). */
	      int loop;                /* OTHERS = msg to anyone else present. */
				       /* Leave msgs blank if not needed.   */
	      say (msg,player);           /* If no receiver, set num2 to player.  */

	      if (strcmp(someone,""))
	      {
			  say (someone,num2);
	      }

	      if (strcmp(others,""))
	      {
			 for (loop = 0; loop <= totalplayers; loop ++)
			 {
				  if (loop != player && loop != num2 && player[loop].location == player[playernum].location)
					 say(others,loop);
			 }
	      }

}


void tell1a(char msg [85], char others[85])
{

	      int loop;
	      say (msg,player);
	      for (loop = 0; loop <= totalplayers; loop ++)
	      {
			if (loop != player && player[loop].location == player[playernum].location)
				say(others,loop);
	      }


}

void tellallpresent(char msg [], int location)
{
/* Say msg to all players in the location */

	      int loop;

	      for (loop = 0; loop < numplayers; loop ++)
	      {
			 if (player[loop].location == location)
				say(msg,loop);
	      }
}
void tell1(char player[85])
{
	      say (player,player);
}


void writemons(char block [20])
{
	FILE *fp;
	char temp [9];
	char temp3 [3];
	int a = 0,b = 0;

	fp = fopen(block,"w");
	if (fp == NULL)
		 exit(2);
	for (a = 0;a <= mon;a++)
	{
		/*  printf("Writing %i %i %i %i %i %i %i %i\n",mons[a].type,mons[a].location,mons[a].life,mons[a].armour,mons[a].skill,mons[a].money,mons[a].mood,mons[a].weapon); */
		 fprintf(fp,"%i %i %i %i %i %i %i %i\n",mons[a].type,mons[a].location,mons[a].life,mons[a].armour,mons[a].skill,mons[a].money,mons[a].mood,mons[a].weapon);
	}
	fprintf(fp,"0 0 0 0 0 0 0 0\n");
	fclose(fp);

}


void writeobjects(char block [20])
{
	FILE *fp;
	int a;
	fp = fopen(block,"w");
	if (fp == NULL)
	      exit(2);
     /*   puts("writing objects");      */
	for (a = 1;a < MAXOBJECTS;a++)
	{
	    /*    printf("%s\n%s\n",objects[a].name,objects[a].desc);  */
		fprintf(fp,"%s\n%s\n",objects[a].name,objects[a].desc);
	  /*      printf("%i %i %i\n",objects[a].weight,objects[a].location,objects[a].dd);   */
		fprintf(fp,"%i %i %i\n",objects[a].weight,objects[a].location,objects[a].dd);
	}
	fprintf(fp,"***\n");
	fclose(fp);
}

void writeplayer()
{
     FILE *fp,*of;
     char temp [30];
     char temp2 [30];
     char temp3 [120];
     int a = 0, b = 0, c = 0, flag = 0,end = 0,hey = 1,gotplayer = 0;
     float d = 0;
     strcpy(temp,"");
     strcpy(temp2,"");
     strcpy(temp3,"");
     flag = rename("player.dat","player.old");
     if (flag)
     {
	   end = 1;
	/*   puts("no existing player.dat file");   */
     }
     else
     {
	  /*  puts("reading old player file");  */
	    of = fopen("player.old","rt");
	    rewind(of);
     }
     fp = fopen("player.dat","wt");
     if (fp == NULL)
	  exit(1);

     do
     {
	 if (flag == 0)
	 {
		fscanf(of,"%[^\n]",temp);           /* read a player name  */
		getc(fp);
		if (strstr(temp,"***") != NULL || strcmp(temp,"") == 0 || strcmp(temp,"-") == 0)
		{
		       if (gotplayer == 1)
			      break;
		       else
			      end = 1;
		}
	 }
     /*    printf("read the name %s.  Your name is %s\n",temp,player[playernum].playername); */
	 a = strncmp(temp,player[playernum].playername,strlen(temp));
     /*    printf("compare = %i\n",a);      */
	 if (strnicmp(temp,player[playernum].playername,strlen(temp)) == 0 || end == 1)
	 {
	      gotplayer = 1;
	      hey = 0;
	      fprintf(fp,"%s\n%s\n%s\n",player[playernum].playername,player[playernum].name,player[playernum].desc);
	      fprintf(fp,"%i\n%s\n%i\n%i\n%i\n%i\n%f\n%i\n%i\n%i\n%i\n%i\n",player[playernum].job,player[playernum].sex,player[playernum].money,player[playernum].food,player[playernum].ds,player[playernum].damage,player[playernum].location,player[playernum].life,player[playernum].species,player[playernum].weapon,player[playernum].armour,player[playernum].weight);
	      fprintf(fp,"%i\n%i\n%f\n",player[playernum].hunger,player[playernum].carry,player[playernum].relative);
	      for (a = 0; a < 6; a++)
		       fprintf(fp,"%i\n",player[playernum].xp[a]);          /*  your xp     */
	      for (a = 0;a < 6;a++)
		       fprintf(fp,"%i\n",player[playernum].scores[a]);      /* your int. score etc */
	      for (a = 0;a < 6;a++)
		       fprintf(fp,"%i\n",player[playernum].skills[a]);     /* your skill points */
	      for (a = 1;a < maxequip;a++)
	      {
		       if (strcmp(equip[a].name[playernum],"") == 0)
			      strcpy(equip[a].name[playernum],"-");

		       fprintf(fp,"%s\n",equip[a].name[playernum]);   /* your equip  */


		       fprintf(fp,"%i\n%i\n",equip[a].weight[playernum],equip[a].dd[playernum]);
	      }
	 /*   puts("written your data");  */
	      if (end == 1)
	      break;
	 }
     /*    printf("Hey = %i (0 = your data)\n",hey); */
	 if (hey)
	       fprintf(fp,"%s\n",temp);
	 fscanf(of,"%[^\n]",temp);
	 if (hey)
	       fprintf(fp,"%s\n",temp);
	 fscanf(of,"%[^\n]",temp3);
	 if (hey)
	       fprintf(fp,"%s\n",temp3);
	 fscanf(of,"%i\n%s\n",&b,temp2);
     /*    printf("%i\n%s\n",b,temp2);  */
	 if (hey)
	       fprintf(fp,"%i\n%s\n",b,temp2);
	 for (a = 0; a <4; a++)
	 {
		  fscanf(of,"%i\n",&c);
	  /*        printf("%i\n",c);        */
		  if (hey)
			 fprintf(fp,"%i\n",c);
	 }
	 fscanf(of,"%f\n",&d);
	 if (hey)
	       fprintf(fp,"%f\n",d);
	 for (a = 0; a < 7; a++)
	 {
		  fscanf(of,"%i\n",&c);
	    /*      printf("%i\n",c);      */
		  if (hey)
			fprintf(fp,"%i\n",c);
	 }
	 fscanf(of,"%f\n",&d);
	 if (hey)
		fprintf(fp,"%f\n",d);
	 for (a = 0; a < 18; a++)
	 {
		  fscanf(of,"%i\n",&c);
	   /*       printf("%i\n",c);        */
		  if (hey)
			fprintf(fp,"%i\n",c);
	 }
	 for (a = 1;a < maxequip;a++)
	 {
		  fscanf(of,"%[^\n]",temp2);
		  getc(of);
	     /*     printf("number %i... name %s\n",a,temp2);  */
		  if (hey)
			fprintf(fp,"%s\n",temp2);

		  fscanf(of,"%i\n%i\n",&b,&c);
	     /*     printf("%i\n%i\n",b,c);         */
		  if (hey)
			fprintf(fp,"%i\n%i\n",b,c);
	 }
	 if (hey == 0)
		  hey = 1;
  /*              puts("at end of do loop");                  */
     } while (1);
 /*    puts("finished loop"); */
     if (flag == 0)
     {
	    fclose(of);
	    remove("player.old");
     }
     fprintf(fp,"***\n");
     fclose(fp);
}


int yesorno(char temp3[80])               /* Get yes or no answer.  1 = yes */
{                                         /* 0 = no.                        */
       char temp[30];
       char temp2[30];
       int a = 0;
       strcpy(temp,"");
       strcpy(temp2,"");
       while (strnicmp(temp,"y",1) != 0 && strnicmp(temp,"n",1) != 0)
	       getinput(temp,temp2,temp3);
       if (strnicmp(temp,"y",1) == 0)
	       a = 1;
       return(a);
}


float playermove(int verb)                     /* Returns 0 if ok           */
{                                           /*   or 1 if impossible      */
     int b = 0, ok = 1;
     float a = 0;
     char temp [13];
     char temp2 [13];
     char temp3 [3];
     strcpy(temp,"");
     strcpy(temp2,"");
     strcpy(temp3,"");
     if (map [(int) player[playernum].relative] [verb] > 0)
     {
	   ok = 0;
	   a = (float) map [(int) player[playernum].relative] [verb];
	   if ( getblock(a) != getblock(player[playernum].location))
	   {

		 writemons(temp);                    /* Write mons & objects */
		 writeobjects(temp);
		 player[playernum].location = a;                  /* Change locations  */
		 readmons(temp,temp2);
		 readobjects();
		 readlocation(temp);
	   }
	   if (player[playernum].location != a)
		 player[playernum].location = a;
     }
     return(ok);
}


void assignhome (enum terraintype terrain, int *location)
{
/* Set location to a location of the given terrain. */


	int loc;

	*location = -1;
	for (loc = 0; loc < MAXLOCATIONS; loc ++)
	{
		if (locations[loc].terrain == terrain)
		{
			if (*location == -1 || (rnd (0,2)) == 1)
				*location = loc;
		}
	}
}

int assignobject(int objecttype,int holdernum,enum ownertype owner)
{
/* Create a new object of the type objecttype and give it to holdernum */
/* Return the object number */
/* If there's no room for new objects in the objects array then return NONE */

	int loop, foundone = FALSE;

	if (objecttype != NONE && numobjects < (MAXOBJECTS - 1))
	{
		numobjects ++;
		for (loop = 0; loop < MAXSHOPSTUFF; loop ++)
		{
			if (shopstuff[loop].type == objecttype)
			{
				if (!foundone || (rnd(0,2)) == 1)
				{
					foundone = TRUE;
					objects[numobjects].type = objecttype;
					objects[numobjects].location = holdernum;
					objects[numobjects].owner = owner;
				}
			}
		}
	}
	if (foundone)
		return (numobjects);
	else
		return (NONE);
}
